# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.sparse.linalg.isolve._iterative, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def cbicgrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = cbicgrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``cbicgrevcom``.\n\nParameters\n----------\nb : input rank-1 array('F') with bounds (n)\nx : input rank-1 array('F') with bounds (n)\nwork : in/output rank-1 array('F') with bounds (6 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('F') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def cbicgstabrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = cbicgstabrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``cbicgstabrevcom``.\n\nParameters\n----------\nb : input rank-1 array('F') with bounds (n)\nx : input rank-1 array('F') with bounds (n)\nwork : in/output rank-1 array('F') with bounds (7 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('F') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def ccgrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = ccgrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``ccgrevcom``.\n\nParameters\n----------\nb : input rank-1 array('F') with bounds (n)\nx : input rank-1 array('F') with bounds (n)\nwork : in/output rank-1 array('F') with bounds (4 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('F') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def ccgsrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = ccgsrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``ccgsrevcom``.\n\nParameters\n----------\nb : input rank-1 array('F') with bounds (n)\nx : input rank-1 array('F') with bounds (n)\nwork : in/output rank-1 array('F') with bounds (7 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('F') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def cgmresrevcom(b, x, restrt, work, work2, iter, resid, info, ndx1, ndx2, ijob, tol) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = cgmresrevcom(b,x,restrt,work,work2,iter,resid,info,ndx1,ndx2,ijob,tol)\n\nWrapper for ``cgmresrevcom``.\n\nParameters\n----------\nb : input rank-1 array('F') with bounds (n)\nx : input rank-1 array('F') with bounds (n)\nrestrt : input int\nwork : in/output rank-1 array('F') with bounds (ldw*(6+restrt))\nwork2 : in/output rank-1 array('F') with bounds (ldw2*(2*restrt+2))\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\ntol : input float\n\nReturns\n-------\nx : rank-1 array('F') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def cqmrrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = cqmrrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``cqmrrevcom``.\n\nParameters\n----------\nb : input rank-1 array('F') with bounds (n)\nx : input rank-1 array('F') with bounds (n)\nwork : in/output rank-1 array('F') with bounds (11 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('F') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def dbicgrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = dbicgrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``dbicgrevcom``.\n\nParameters\n----------\nb : input rank-1 array('d') with bounds (n)\nx : input rank-1 array('d') with bounds (n)\nwork : in/output rank-1 array('d') with bounds (6 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('d') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def dbicgstabrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = dbicgstabrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``dbicgstabrevcom``.\n\nParameters\n----------\nb : input rank-1 array('d') with bounds (n)\nx : input rank-1 array('d') with bounds (n)\nwork : in/output rank-1 array('d') with bounds (7 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('d') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def dcgrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = dcgrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``dcgrevcom``.\n\nParameters\n----------\nb : input rank-1 array('d') with bounds (n)\nx : input rank-1 array('d') with bounds (n)\nwork : in/output rank-1 array('d') with bounds (4 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('d') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def dcgsrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = dcgsrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``dcgsrevcom``.\n\nParameters\n----------\nb : input rank-1 array('d') with bounds (n)\nx : input rank-1 array('d') with bounds (n)\nwork : in/output rank-1 array('d') with bounds (7 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('d') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def dgmresrevcom(b, x, restrt, work, work2, iter, resid, info, ndx1, ndx2, ijob, tol) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = dgmresrevcom(b,x,restrt,work,work2,iter,resid,info,ndx1,ndx2,ijob,tol)\n\nWrapper for ``dgmresrevcom``.\n\nParameters\n----------\nb : input rank-1 array('d') with bounds (n)\nx : input rank-1 array('d') with bounds (n)\nrestrt : input int\nwork : in/output rank-1 array('d') with bounds (ldw*(6+restrt))\nwork2 : in/output rank-1 array('d') with bounds (ldw2*(2*restrt+2))\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\ntol : input float\n\nReturns\n-------\nx : rank-1 array('d') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def dqmrrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = dqmrrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``dqmrrevcom``.\n\nParameters\n----------\nb : input rank-1 array('d') with bounds (n)\nx : input rank-1 array('d') with bounds (n)\nwork : in/output rank-1 array('d') with bounds (11 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('d') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def sbicgrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = sbicgrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``sbicgrevcom``.\n\nParameters\n----------\nb : input rank-1 array('f') with bounds (n)\nx : input rank-1 array('f') with bounds (n)\nwork : in/output rank-1 array('f') with bounds (6 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('f') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def sbicgstabrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = sbicgstabrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``sbicgstabrevcom``.\n\nParameters\n----------\nb : input rank-1 array('f') with bounds (n)\nx : input rank-1 array('f') with bounds (n)\nwork : in/output rank-1 array('f') with bounds (7 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('f') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def scgrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = scgrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``scgrevcom``.\n\nParameters\n----------\nb : input rank-1 array('f') with bounds (n)\nx : input rank-1 array('f') with bounds (n)\nwork : in/output rank-1 array('f') with bounds (4 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('f') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def scgsrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = scgsrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``scgsrevcom``.\n\nParameters\n----------\nb : input rank-1 array('f') with bounds (n)\nx : input rank-1 array('f') with bounds (n)\nwork : in/output rank-1 array('f') with bounds (7 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('f') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def sgmresrevcom(b, x, restrt, work, work2, iter, resid, info, ndx1, ndx2, ijob, tol) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = sgmresrevcom(b,x,restrt,work,work2,iter,resid,info,ndx1,ndx2,ijob,tol)\n\nWrapper for ``sgmresrevcom``.\n\nParameters\n----------\nb : input rank-1 array('f') with bounds (n)\nx : input rank-1 array('f') with bounds (n)\nrestrt : input int\nwork : in/output rank-1 array('f') with bounds (ldw*(6+restrt))\nwork2 : in/output rank-1 array('f') with bounds (ldw2*(2*restrt+2))\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\ntol : input float\n\nReturns\n-------\nx : rank-1 array('f') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def sqmrrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = sqmrrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``sqmrrevcom``.\n\nParameters\n----------\nb : input rank-1 array('f') with bounds (n)\nx : input rank-1 array('f') with bounds (n)\nwork : in/output rank-1 array('f') with bounds (11 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('f') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : float\nsclr2 : float\nijob : int\n"
    ...

def zbicgrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = zbicgrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``zbicgrevcom``.\n\nParameters\n----------\nb : input rank-1 array('D') with bounds (n)\nx : input rank-1 array('D') with bounds (n)\nwork : in/output rank-1 array('D') with bounds (6 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('D') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def zbicgstabrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = zbicgstabrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``zbicgstabrevcom``.\n\nParameters\n----------\nb : input rank-1 array('D') with bounds (n)\nx : input rank-1 array('D') with bounds (n)\nwork : in/output rank-1 array('D') with bounds (7 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('D') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def zcgrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = zcgrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``zcgrevcom``.\n\nParameters\n----------\nb : input rank-1 array('D') with bounds (n)\nx : input rank-1 array('D') with bounds (n)\nwork : in/output rank-1 array('D') with bounds (4 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('D') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def zcgsrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = zcgsrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``zcgsrevcom``.\n\nParameters\n----------\nb : input rank-1 array('D') with bounds (n)\nx : input rank-1 array('D') with bounds (n)\nwork : in/output rank-1 array('D') with bounds (7 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('D') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def zgmresrevcom(b, x, restrt, work, work2, iter, resid, info, ndx1, ndx2, ijob, tol) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = zgmresrevcom(b,x,restrt,work,work2,iter,resid,info,ndx1,ndx2,ijob,tol)\n\nWrapper for ``zgmresrevcom``.\n\nParameters\n----------\nb : input rank-1 array('D') with bounds (n)\nx : input rank-1 array('D') with bounds (n)\nrestrt : input int\nwork : in/output rank-1 array('D') with bounds (ldw*(6+restrt))\nwork2 : in/output rank-1 array('D') with bounds (ldw2*(2*restrt+2))\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\ntol : input float\n\nReturns\n-------\nx : rank-1 array('D') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def zqmrrevcom(b, x, work, iter, resid, info, ndx1, ndx2, ijob) -> typing.Any:
    "x,iter,resid,info,ndx1,ndx2,sclr1,sclr2,ijob = zqmrrevcom(b,x,work,iter,resid,info,ndx1,ndx2,ijob)\n\nWrapper for ``zqmrrevcom``.\n\nParameters\n----------\nb : input rank-1 array('D') with bounds (n)\nx : input rank-1 array('D') with bounds (n)\nwork : in/output rank-1 array('D') with bounds (11 * ldw)\niter : input int\nresid : input float\ninfo : input int\nndx1 : input int\nndx2 : input int\nijob : input int\n\nReturns\n-------\nx : rank-1 array('D') with bounds (n)\niter : int\nresid : float\ninfo : int\nndx1 : int\nndx2 : int\nsclr1 : complex\nsclr2 : complex\nijob : int\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from ..classes.digraph import DiGraph
from numpy.typing import ArrayLike
from itertools import combinations

from ..classes.graph import Graph
from ..algorithms.simple_paths import is_simple_path as is_path
from ..utils import arbitrary_element, not_implemented_for, py_random_state

__all__ = [
    "hamiltonian_path",
    "is_reachable",
    "is_strongly_connected",
    "is_tournament",
    "random_tournament",
    "score_sequence",
]

def index_satisfying(iterable, condition): ...
def is_tournament(G: Graph) -> bool: ...
def hamiltonian_path(G: Graph) -> ArrayLike: ...
@py_random_state(1)
def random_tournament(n: int, seed=None) -> DiGraph: ...
def score_sequence(G: Graph) -> ArrayLike: ...
def tournament_matrix(G: Graph): ...
def is_reachable(G: Graph, s, t) -> bool: ...
def is_strongly_connected(G: Graph) -> bool: ...



from typing import Callable, Literal, Sequence, TypeVar
from scipy.special import kv as kv, gamma
from collections import namedtuple
from scipy.spatial.distance import (
    pdist as pdist,
    cdist as cdist,
    squareform as squareform,
)
from abc import ABCMeta, abstractmethod
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from numpy import ndarray
from ..base import clone as clone
from inspect import signature as signature
from ..metrics.pairwise import pairwise_kernels as pairwise_kernels
from .._typing import ArrayLike, MatrixLike, Float

Kernel_Self = TypeVar("Kernel_Self", bound="Kernel")
Hyperparameter_Self = TypeVar("Hyperparameter_Self", bound="Hyperparameter")

import math

import numpy as np

import warnings


class Hyperparameter(
    namedtuple(
        "Hyperparameter", ("name", "value_type", "bounds", "n_elements", "fixed")
    )
):
    fixed: bool = ...
    n_elements: int = ...
    bounds: tuple[float, float] | str = ...
    value_type: str = ...
    name: str = ...

    # A raw namedtuple is very memory efficient as it packs the attributes
    # in a struct to get rid of the __dict__ of attributes in particular it
    # does not copy the string for the keys on each instance.
    # By deriving a namedtuple class just to introduce the __init__ method we
    # would also reintroduce the __dict__ on the instance. By telling the
    # Python interpreter that this subclass uses static __slots__ instead of
    # dynamic attributes. Furthermore we don't need any additional slot in the
    # subclass so we set __slots__ to the empty tuple.
    __slots__ = ...

    def __new__(
        cls,
        name: str,
        value_type: str,
        bounds: ndarray | str | tuple[float, int] | tuple[float, float],
        n_elements: int = 1,
        fixed=None,
    ) -> Hyperparameter:
        ...

    # This is mainly a testing utility to check that two hyperparameters
    # are equal.
    def __eq__(self, other) -> bool:
        ...


class Kernel(metaclass=ABCMeta):
    def get_params(self, deep: bool = True) -> dict:
        ...

    def set_params(self: Kernel_Self, **params) -> Kernel_Self:
        ...

    def clone_with_theta(self, theta: ArrayLike):
        ...

    def n_dims(self) -> int:
        ...

    def hyperparameters(self) -> list[Hyperparameter]:
        ...

    @property
    def theta(self) -> ndarray:
        ...

    @theta.setter
    def theta(self, theta) -> ndarray:
        ...

    @property
    def bounds(self) -> ndarray:
        ...

    def __add__(self, b) -> Sum:
        ...

    def __radd__(self, b):
        ...

    def __mul__(self, b) -> Product:
        ...

    def __rmul__(self, b: float | int) -> Product:
        ...

    def __pow__(self, b: int) -> Exponentiation:
        ...

    def __eq__(self, b: str) -> bool:
        ...

    def __repr__(self) -> str:
        ...

    @abstractmethod
    def __call__(self, X, Y=None, eval_gradient: bool = False):
        ...

    @abstractmethod
    def diag(self, X: ArrayLike) -> ndarray:
        ...

    @abstractmethod
    def is_stationary(self):
        ...

    def requires_vector_input(self) -> bool:
        ...


class NormalizedKernelMixin:
    def diag(self, X: MatrixLike) -> ndarray:
        ...


class StationaryKernelMixin:
    def is_stationary(self):
        ...


class GenericKernelMixin:
    def requires_vector_input(self) -> bool:
        ...


class CompoundKernel(Kernel):
    def __init__(self, kernels: Sequence[Kernel]) -> None:
        ...

    def get_params(self, deep: bool = True) -> dict:
        ...

    @property
    def theta(self) -> ndarray:
        ...

    @theta.setter
    def theta(self, theta) -> ndarray:
        ...

    @property
    def bounds(self) -> ndarray:
        ...

    def __call__(
        self,
        X: MatrixLike | ArrayLike,
        Y: None | MatrixLike | ArrayLike = None,
        eval_gradient: bool = False,
    ) -> tuple[ndarray, ndarray]:
        ...

    def __eq__(self, b) -> bool:
        ...

    def is_stationary(self):
        ...

    def requires_vector_input(self):
        ...

    def diag(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...


class KernelOperator(Kernel):
    def __init__(self, k1, k2) -> None:
        ...

    def get_params(self, deep: bool = True) -> dict:
        ...

    def hyperparameters(self) -> list[Hyperparameter]:
        ...

    @property
    def theta(self) -> ndarray:
        ...

    @theta.setter
    def theta(self, theta) -> ndarray:
        ...

    @property
    def bounds(self) -> ndarray:
        ...

    def __eq__(self, b) -> bool:
        ...

    def is_stationary(self):
        ...

    def requires_vector_input(self) -> bool:
        ...


class Sum(KernelOperator):
    def __call__(
        self,
        X: MatrixLike | ArrayLike,
        Y: None | MatrixLike | ArrayLike = None,
        eval_gradient: bool = False,
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def diag(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def __repr__(self) -> str:
        ...


class Product(KernelOperator):
    def __call__(
        self,
        X: MatrixLike | ArrayLike,
        Y: Sequence | None | MatrixLike = None,
        eval_gradient: bool = False,
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def diag(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def __repr__(self) -> str:
        ...


class Exponentiation(Kernel):
    def __init__(self, kernel: DotProduct | Kernel, exponent: Float) -> None:
        ...

    def get_params(self, deep: bool = True) -> dict[str, DotProduct | int] | dict:
        ...

    def hyperparameters(self) -> list[Hyperparameter]:
        ...

    @property
    def theta(self) -> ndarray:
        ...

    @theta.setter
    def theta(self, theta) -> ndarray:
        ...

    @property
    def bounds(self) -> ndarray:
        ...

    def __eq__(self, b) -> bool:
        ...

    def __call__(
        self,
        X: MatrixLike | ArrayLike,
        Y: Sequence | None | MatrixLike = None,
        eval_gradient: bool = False,
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def diag(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def __repr__(self) -> str:
        ...

    def is_stationary(self):
        ...

    def requires_vector_input(self) -> bool:
        ...


class ConstantKernel(StationaryKernelMixin, GenericKernelMixin, Kernel):
    def __init__(
        self,
        constant_value: Float = 1.0,
        constant_value_bounds: str | tuple[float, float] = ...,
    ) -> None:
        ...

    def hyperparameter_constant_value(self) -> Hyperparameter:
        ...

    def __call__(
        self,
        X: MatrixLike | ArrayLike,
        Y: None | MatrixLike | ArrayLike = None,
        eval_gradient: bool = False,
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def diag(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def __repr__(self) -> str:
        ...


class WhiteKernel(StationaryKernelMixin, GenericKernelMixin, Kernel):
    def __init__(
        self,
        noise_level: Float = 1.0,
        noise_level_bounds: str | tuple[float, float] = ...,
    ) -> None:
        ...

    def hyperparameter_noise_level(self) -> Hyperparameter:
        ...

    def __call__(
        self,
        X: MatrixLike | ArrayLike,
        Y: None | MatrixLike | ArrayLike = None,
        eval_gradient: bool = False,
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def diag(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def __repr__(self) -> str:
        ...


class RBF(StationaryKernelMixin, NormalizedKernelMixin, Kernel):
    def __init__(
        self,
        length_scale: ArrayLike | Float = 1.0,
        length_scale_bounds: str | tuple[float, float] = ...,
    ) -> None:
        ...

    def anisotropic(self) -> bool:
        ...

    def hyperparameter_length_scale(self) -> Hyperparameter:
        ...

    def __call__(
        self, X: MatrixLike, Y: None | MatrixLike = None, eval_gradient: bool = False
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def __repr__(self) -> str:
        ...


class Matern(RBF):
    def __init__(
        self,
        length_scale: ArrayLike | Float = 1.0,
        length_scale_bounds: str | tuple[float, float] = ...,
        nu: Float = 1.5,
    ) -> None:
        ...

    def __call__(
        self, X: MatrixLike, Y: None | MatrixLike = None, eval_gradient: bool = False
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def __repr__(self) -> str:
        ...


class RationalQuadratic(StationaryKernelMixin, NormalizedKernelMixin, Kernel):
    def __init__(
        self,
        length_scale: Float = 1.0,
        alpha: Float = 1.0,
        length_scale_bounds: str | tuple[float, float] = ...,
        alpha_bounds: str | tuple[float, float] = ...,
    ) -> None:
        ...

    def hyperparameter_length_scale(self) -> Hyperparameter:
        ...

    def hyperparameter_alpha(self) -> Hyperparameter:
        ...

    def __call__(
        self, X: MatrixLike, Y: None | MatrixLike = None, eval_gradient: bool = False
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def __repr__(self) -> str:
        ...


class ExpSineSquared(StationaryKernelMixin, NormalizedKernelMixin, Kernel):
    def __init__(
        self,
        length_scale: Float = 1.0,
        periodicity: Float = 1.0,
        length_scale_bounds: str | tuple[float, float] = ...,
        periodicity_bounds: str | tuple[float, float] = ...,
    ) -> None:
        ...

    def hyperparameter_length_scale(self) -> Hyperparameter:
        ...

    def hyperparameter_periodicity(self) -> Hyperparameter:
        ...

    def __call__(
        self, X: MatrixLike, Y: None | MatrixLike = None, eval_gradient: bool = False
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def __repr__(self) -> str:
        ...


class DotProduct(Kernel):
    def __init__(
        self, sigma_0: Float = 1.0, sigma_0_bounds: str | tuple[float, float] = ...
    ) -> None:
        ...

    def hyperparameter_sigma_0(self) -> Hyperparameter:
        ...

    def __call__(
        self, X: MatrixLike, Y: None | MatrixLike = None, eval_gradient: bool = False
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def diag(self, X: MatrixLike) -> ndarray:
        ...

    def is_stationary(self):
        ...

    def __repr__(self) -> str:
        ...


class PairwiseKernel(Kernel):
    def __init__(
        self,
        gamma: Float = 1.0,
        gamma_bounds: str | tuple[float, float] = ...,
        metric: Literal[
            "linear",
            "additive_chi2",
            "chi2",
            "poly",
            "polynomial",
            "rbf",
            "laplacian",
            "sigmoid",
            "cosine",
            "linear",
        ]
        | Callable = "linear",
        pairwise_kernels_kwargs: None | dict = None,
    ) -> None:
        ...

    def hyperparameter_gamma(self):
        ...

    def __call__(
        self, X: MatrixLike, Y: None | MatrixLike = None, eval_gradient: bool = False
    ) -> tuple[ndarray, ndarray]:
        ...

    def diag(self, X: MatrixLike) -> ndarray:
        ...

    def is_stationary(self):
        ...

    def __repr__(self) -> str:
        ...



from typing import Any, ClassVar, Iterator, Literal, TypeVar
from numpy.random import RandomState
from scipy.special import xlogy as xlogy
from abc import ABCMeta, abstractmethod
from ..tree import (
    DecisionTreeClassifier as DecisionTreeClassifier,
    DecisionTreeRegressor as DecisionTreeRegressor,
)
from ..base import ClassifierMixin, RegressorMixin, BaseEstimator
from ._base import BaseEnsemble
from numpy import ndarray
from ..utils.extmath import softmax as softmax, stable_cumsum as stable_cumsum
from ..utils._param_validation import (
    HasMethods as HasMethods,
    Interval as Interval,
    StrOptions as StrOptions,
)
from numbers import Integral as Integral, Real as Real
from ..metrics import accuracy_score as accuracy_score, r2_score as r2_score
from ..base import is_classifier as is_classifier, is_regressor as is_regressor
from .._typing import MatrixLike, ArrayLike, Int, Float
from ..utils import check_random_state as check_random_state
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    has_fit_parameter as has_fit_parameter,
)

BaseWeightBoosting_Self = TypeVar("BaseWeightBoosting_Self", bound="BaseWeightBoosting")

import numpy as np

import warnings

__all__ = [
    "AdaBoostClassifier",
    "AdaBoostRegressor",
]


class BaseWeightBoosting(BaseEnsemble, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        estimator=None,
        *,
        n_estimators: int = 50,
        estimator_params=...,
        learning_rate: float = 1.0,
        random_state=None,
        base_estimator: str = "deprecated",
    ) -> None:
        ...

    def fit(
        self: BaseWeightBoosting_Self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> BaseWeightBoosting_Self:
        ...

    def staged_score(
        self,
        X: MatrixLike | ArrayLike,
        y: ArrayLike,
        sample_weight: None | ArrayLike = None,
    ):
        ...

    @property
    def feature_importances_(self) -> ndarray:
        ...


class AdaBoostClassifier(ClassifierMixin, BaseWeightBoosting):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    feature_importances_: ndarray = ...
    estimator_errors_: ndarray = ...
    estimator_weights_: ndarray = ...
    n_classes_: int = ...
    classes_: ndarray = ...
    estimators_: list[ClassifierMixin] = ...
    base_estimator_: BaseEstimator = ...
    estimator_: BaseEstimator = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimator: Any = None,
        *,
        n_estimators: Int = 50,
        learning_rate: Float = 1.0,
        algorithm: Literal["SAMME", "SAMME.R", "SAMME.R"] = "SAMME.R",
        random_state: RandomState | None | Int = None,
        base_estimator: Any = "deprecated",
    ) -> None:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def staged_predict(self, X: MatrixLike) -> Iterator[ndarray]:
        ...

    def decision_function(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def staged_decision_function(self, X: MatrixLike | ArrayLike) -> Iterator[ndarray]:
        ...

    def predict_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def staged_predict_proba(self, X: MatrixLike | ArrayLike):
        ...

    def predict_log_proba(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...


class AdaBoostRegressor(RegressorMixin, BaseWeightBoosting):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    feature_importances_: ndarray = ...
    estimator_errors_: ndarray = ...
    estimator_weights_: ndarray = ...
    estimators_: list[RegressorMixin] = ...
    base_estimator_: BaseEstimator = ...
    estimator_: BaseEstimator = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        estimator: Any = None,
        *,
        n_estimators: Int = 50,
        learning_rate: Float = 1.0,
        loss: Literal["linear", "square", "exponential", "linear"] = "linear",
        random_state: RandomState | None | Int = None,
        base_estimator: Any = "deprecated",
    ) -> None:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def staged_predict(self, X: MatrixLike | ArrayLike):
        ...



                 
import numpy as np
from typing import Iterable, Any
from scipy.sparse import csr_matrix


def get_valid_metric_ids(L: Iterable) -> list[str]: ...


class DistanceMetric():

    def __init__(self)->None: ...

    @classmethod
    def get_metric(cls, metric: str, **kwargs) -> DistanceMetric: ...

    def dist(self, x1, x2, size): ...
    def rdist(self, x1, x2, size): ...
    def pdist(self, X, D) -> int: ...
    def cdist(self, X, Y, D) -> int: ...
    def dist_csr(self, x1_data, x1_indices, x2_data, x2_indices, x1_start, x1_end, x2_start, x2_end, size): ...
    def rdist_csr(self, x1_data, x1_indices, x2_data, x2_indices, x1_start, x1_end, x2_start, x2_end, size): ...
    def pdist_csr(self, x1_data, x1_indices, x1_indptr, size, D)->int: ...
    def cdist_csr(self, x1_data, x1_indices, x1_indptr, x2_data, x2_indices, x2_indptr, size, D)->int: ...
    def rdist_to_dist(self, rdist: float) -> float: ...
    def dist_to_rdist(self, dist: float) -> float: ...
    def pairwise(self, X: np.ndarray|csr_matrix, Y: np.ndarray|csr_matrix|None=None)->np.ndarray: ...

    
class DistanceMetric32():

    def __init__(self)->None: ...

    @classmethod
    def get_metric(cls, metric: str, **kwargs) -> DistanceMetric: ...

    def dist(self, x1, x2, size): ...
    def rdist(self, x1, x2, size): ...
    def pdist(self, X, D) -> int: ...
    def cdist(self, X, Y, D) -> int: ...
    def dist_csr(self, x1_data, x1_indices, x2_data, x2_indices, x1_start, x1_end, x2_start, x2_end, size): ...
    def rdist_csr(self, x1_data, x1_indices, x2_data, x2_indices, x1_start, x1_end, x2_start, x2_end, size): ...
    def pdist_csr(self, x1_data, x1_indices, x1_indptr, size, D)->int: ...
    def cdist_csr(self, x1_data, x1_indices, x1_indptr, x2_data, x2_indices, x2_indptr, size, D)->int: ...
    def rdist_to_dist(self, rdist: float) -> float: ...
    def dist_to_rdist(self, dist: float) -> float: ...
    def pairwise(self, X: np.ndarray|csr_matrix, Y: np.ndarray|csr_matrix|None=None)->np.ndarray: ...

class BrayCurtisDistance(DistanceMetric): ...
class BrayCurtisDistance32(DistanceMetric32): ...
class CanberraDistance(DistanceMetric): ...
class CanberraDistance32(DistanceMetric32): ...
class ChebyshevDistance(DistanceMetric): ...
class ChebyshevDistance32(DistanceMetric32): ...
class DiceDistance(DistanceMetric): ...
class DiceDistance32(DistanceMetric32): ...
class EuclideanDistance(DistanceMetric): ...
class EuclideanDistance32(DistanceMetric32): ...
class HammingDistance(DistanceMetric): ...
class HammingDistance32(DistanceMetric32): ...
class HaversineDistance(DistanceMetric): ...
class HaversineDistance32(DistanceMetric32): ...
class JaccardDistance(DistanceMetric): ...
class JaccardDistance32(DistanceMetric32): ...
class KulsinskiDistance(DistanceMetric): ...
class KulsinskiDistance32(DistanceMetric32): ...
class MahalanobisDistance(DistanceMetric): ...
class MahalanobisDistance32(DistanceMetric32): ...
class ManhattanDistance(DistanceMetric): ...
class ManhattanDistance32(DistanceMetric32): ...
class MatchingDistance(DistanceMetric): ...
class MatchingDistance32(DistanceMetric32): ...
class MinkowskiDistance(DistanceMetric): ...
class MinkowskiDistance32(DistanceMetric32): ...
class PyFuncDistance(DistanceMetric): ...
class PyFuncDistance32(DistanceMetric32): ...
class RogersTanimotoDistance(DistanceMetric): ...
class RogersTanimotoDistance32(DistanceMetric32): ...
class RussellRaoDistance(DistanceMetric): ...
class RussellRaoDistance32(DistanceMetric32): ...
class SEuclideanDistance(DistanceMetric): ...
class SEuclideanDistance32(DistanceMetric32): ...
class SokalMichenerDistance(DistanceMetric): ...
class SokalMichenerDistance32(DistanceMetric32): ...
class SokalSneathDistance(DistanceMetric): ...
class SokalSneathDistance32(DistanceMetric32): ...
class WMinkowskiDistance(DistanceMetric): ...
class WMinkowskiDistance32(DistanceMetric32): ...


METRIC_MAPPING: dict[str, Any]





from typing import Literal
from ..base import BaseEstimator
from ._available_if import available_if as available_if
from .._config import get_config as get_config
from functools import wraps as wraps
from scipy.sparse import issparse as issparse
from . import check_pandas_support as check_pandas_support


class _SetOutputMixin:
    def __init_subclass__(
        cls, auto_wrap_output_keys: None | tuple[str] = ..., **kwargs
    ) -> None:
        ...

    def set_output(
        self, *, transform: None | Literal["default", "pandas"] = None
    ) -> BaseEstimator:
        ...



from typing import Literal
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np

from . import LineVisual

class MeshNormalsVisual(LineVisual):
    def __init__(
        self,
        meshdata=None,
        primitive: Literal["face", "vertex"] = "face",
        length: float | ArrayLike | None = None,
        length_method: Literal["median_edge", "max_extent"] = "median_edge",
        length_scale: float = 1.0,
        **kwargs,
    ): ...
    def set_data(
        self,
        meshdata=None,
        primitive: Literal["face", "vertex"] = "face",
        length: float | ArrayLike | None = None,
        length_method: Literal["median_edge", "max_extent"] = "median_edge",
        length_scale: float = 1.0,
        **kwargs,
    ): ...



class MPLPlot:
    orientation: str | None = ...
    data = ...
    by = ...
    kind = ...
    sort_columns = ...
    subplots = ...
    sharex: bool = ...
    sharey = ...
    figsize = ...
    layout = ...
    xticks = ...
    yticks = ...
    xlim = ...
    ylim = ...
    title = ...
    use_index = ...
    fontsize = ...
    rot = ...
    grid = ...
    legend = ...
    legend_handles = ...
    legend_labels = ...
    ax = ...
    fig = ...
    axes = ...
    errors = ...
    secondary_y = ...
    colormap = ...
    table = ...
    include_bool = ...
    kwds = ...
    def __init__(
        self,
        data,
        kind=...,
        by=...,
        subplots: bool = ...,
        sharex=...,
        sharey: bool = ...,
        use_index: bool = ...,
        figsize=...,
        grid=...,
        legend: bool = ...,
        rot=...,
        ax=...,
        fig=...,
        title=...,
        xlim=...,
        ylim=...,
        xticks=...,
        yticks=...,
        sort_columns: bool = ...,
        fontsize=...,
        secondary_y: bool = ...,
        colormap=...,
        table: bool = ...,
        layout=...,
        include_bool: bool = ...,
        **kwds,
    ) -> None: ...
    @property
    def nseries(self): ...
    def draw(self) -> None: ...
    def generate(self) -> None: ...
    @property
    def result(self): ...
    @property
    def legend_title(self): ...
    def plt(self): ...
    @classmethod
    def get_default_ax(cls, ax) -> None: ...
    def on_right(self, i): ...

class PlanePlot(MPLPlot):
    x = ...
    y = ...
    def __init__(self, data, x, y, **kwargs) -> None: ...
    @property
    def nseries(self): ...

class ScatterPlot(PlanePlot):
    c = ...
    def __init__(self, data, x, y, s=..., c=..., **kwargs) -> None: ...

class HexBinPlot(PlanePlot):
    C = ...
    def __init__(self, data, x, y, C=..., **kwargs) -> None: ...

class LinePlot(MPLPlot):
    orientation: str = ...
    data = ...
    x_compat = ...
    def __init__(self, data, **kwargs) -> None: ...

class AreaPlot(LinePlot):
    def __init__(self, data, **kwargs) -> None: ...

class BarPlot(MPLPlot):
    orientation: str = ...
    bar_width = ...
    tick_pos = ...
    bottom = ...
    left = ...
    log = ...
    tickoffset = ...
    lim_offset = ...
    ax_pos = ...
    def __init__(self, data, **kwargs) -> None: ...

class BarhPlot(BarPlot):
    orientation: str = ...

class PiePlot(MPLPlot):
    def __init__(self, data, kind=..., **kwargs) -> None: ...



from typing import List, Optional, Union, overload, Literal
from warnings import warn
from contextlib import closing as closing
from tempfile import TemporaryDirectory as TemporaryDirectory
from urllib.error import HTTPError as HTTPError, URLError as URLError
from os.path import join as join
from urllib.request import urlopen as urlopen, Request as Request
from functools import wraps as wraps
from ._arff_parser import load_arff_from_gzip_file as load_arff_from_gzip_file
from ..utils import Bunch, check_pandas_support as check_pandas_support
from . import get_data_home as get_data_home
import gzip
import hashlib
import json
import os
import shutil
import time

import numpy as np

__all__ = ["fetch_openml"]

_OPENML_PREFIX: str = ...
_SEARCH_NAME: str = ...
_DATA_INFO: str = ...
_DATA_FEATURES: str = ...
_DATA_QUALITIES: str = ...
_DATA_FILE: str = ...

OpenmlQualitiesType = ...
OpenmlFeaturesType = ...


class OpenMLError(ValueError):
    ...


@overload
def fetch_openml(
    name: Optional[str] = None,
    *,
    version: Union[str, int] = "active",
    data_id: Optional[int] = None,
    data_home: Optional[str] = None,
    target_column: Optional[Union[str, List]] = "default-target",
    cache: bool = True,
    return_X_y : Literal[False] = ...,
    as_frame: Union[str, bool] = "auto",
    n_retries: int = 3,
    delay: float = 1.0,
    parser: Optional[str] = "warn",
) -> Bunch:
    ...
@overload
def fetch_openml(
    name: Optional[str] = None,
    *,
    version: Union[str, int] = "active",
    data_id: Optional[int] = None,
    data_home: Optional[str] = None,
    target_column: Optional[Union[str, List]] = "default-target",
    cache: bool = True,
    return_X_y : Literal[True] = ...,
    as_frame: Union[str, bool] = "auto",
    n_retries: int = 3,
    delay: float = 1.0,
    parser: Optional[str] = "warn",
) -> tuple[Bunch, tuple]:
    ...



"""
This type stub file was generated by pyright.
"""

from pdb import Pdb as OldPdb

"""
Pdb debugger class.


This is an extension to PDB which adds a number of new features.
Note that there is also the `IPython.terminal.debugger` class which provides UI
improvements.

We also strongly recommend to use this via the `ipdb` package, which provides
extra configuration options.

Among other things, this subclass of PDB:
 - supports many IPython magics like pdef/psource
 - hide frames in tracebacks based on `__tracebackhide__`
 - allows to skip frames based on `__debuggerskip__`

The skipping and hiding frames are configurable via the `skip_predicates`
command.

By default, frames from readonly files will be hidden, frames containing
``__tracebackhide__=True`` will be hidden.

Frames containing ``__debuggerskip__`` will be stepped over, frames who's parent
frames value of ``__debuggerskip__`` is ``True`` will be skipped.

    >>> def helpers_helper():
    ...     pass
    ...
    ... def helper_1():
    ...     print("don't step in me")
    ...     helpers_helpers() # will be stepped over unless breakpoint set.
    ...
    ...
    ... def helper_2():
    ...     print("in me neither")
    ...

One can define a decorator that wraps a function between the two helpers:

    >>> def pdb_skipped_decorator(function):
    ...
    ...
    ...     def wrapped_fn(*args, **kwargs):
    ...         __debuggerskip__ = True
    ...         helper_1()
    ...         __debuggerskip__ = False
    ...         result = function(*args, **kwargs)
    ...         __debuggerskip__ = True
    ...         helper_2()
    ...         # setting __debuggerskip__ to False again is not necessary
    ...         return result
    ...
    ...     return wrapped_fn

When decorating a function, ipdb will directly step into ``bar()`` by
default:

    >>> @foo_decorator
    ... def bar(x, y):
    ...     return x * y


You can toggle the behavior with

    ipdb> skip_predicates debuggerskip false

or configure it in your ``.pdbrc``



License
-------

Modified from the standard pdb.Pdb class to avoid including readline, so that
the command line completion of other programs which include this isn't
damaged.

In the future, this class will be expanded with improvements over the standard
pdb.

The original code in this file is mainly lifted out of cmd.py in Python 2.2,
with minor changes. Licensing should therefore be under the standard Python
terms.  For details on the PSF (Python Software Foundation) standard license,
see:

https://docs.python.org/2/license.html


All the changes since then are under the same license as IPython.

"""
__skip_doctest__ = ...
prompt = ...
DEBUGGERSKIP = ...
def make_arrow(pad):
    """generate the leading arrow in front of traceback or debugger"""
    ...

def BdbQuit_excepthook(et, ev, tb, excepthook=...):
    """Exception hook which handles `BdbQuit` exceptions.

    All other exceptions are processed using the `excepthook`
    parameter.
    """
    ...

def BdbQuit_IPython_excepthook(self, et, ev, tb, tb_offset=...):
    ...

RGX_EXTRA_INDENT = ...
def strip_indentation(multiline_string):
    ...

def decorate_fn_with_doc(new_fn, old_fn, additional_text=...): # -> (*args: Unknown, **kw: Unknown) -> Unknown:
    """Make new_fn have old_fn's doc string. This is particularly useful
    for the ``do_...`` commands that hook into the help system.
    Adapted from from a comp.lang.python posting
    by Duncan Booth."""
    ...

class Pdb(OldPdb):
    """Modified Pdb class, does not load readline.

    for a standalone version that uses prompt_toolkit, see
    `IPython.terminal.debugger.TerminalPdb` and
    `IPython.terminal.debugger.set_trace()`


    This debugger can hide and skip frames that are tagged according to some predicates.
    See the `skip_predicates` commands.

    """
    default_predicates = ...
    def __init__(self, completekey=..., stdin=..., stdout=..., context=..., **kwargs) -> None:
        """Create a new IPython debugger.

        Parameters
        ----------
        completekey : default None
            Passed to pdb.Pdb.
        stdin : default None
            Passed to pdb.Pdb.
        stdout : default None
            Passed to pdb.Pdb.
        context : int
            Number of lines of source code context to show when
            displaying stacktrace information.
        **kwargs
            Passed to pdb.Pdb.

        Notes
        -----
        The possibilities are python version dependent, see the python
        docs for more info.
        """
        ...
    
    def set_colors(self, scheme): # -> None:
        """Shorthand access to the color table scheme selector method."""
        ...
    
    def set_trace(self, frame=...): # -> None:
        ...
    
    def hidden_frames(self, stack):
        """
        Given an index in the stack return whether it should be skipped.

        This is used in up/down and where to skip frames.
        """
        ...
    
    def interaction(self, frame, traceback): # -> None:
        ...
    
    def precmd(self, line):
        """Perform useful escapes on the command before it is executed."""
        ...
    
    def new_do_frame(self, arg): # -> None:
        ...
    
    def new_do_quit(self, arg):
        ...
    
    do_quit = ...
    def new_do_restart(self, arg):
        """Restart command. In the context of ipython this is exactly the same
        thing as 'quit'."""
        ...
    
    def print_stack_trace(self, context=...): # -> None:
        ...
    
    def print_stack_entry(self, frame_lineno, prompt_prefix=..., context=...): # -> None:
        ...
    
    def format_stack_entry(self, frame_lineno, lprefix=..., context=...):
        ...
    
    def print_list_lines(self, filename, first, last): # -> None:
        """The printing (as opposed to the parsing part of a 'list'
        command."""
        ...
    
    def do_skip_predicates(self, args): # -> None:
        """
        Turn on/off individual predicates as to whether a frame should be hidden/skip.

        The global option to skip (or not) hidden frames is set with skip_hidden

        To change the value of a predicate

            skip_predicates key [true|false]

        Call without arguments to see the current values.

        To permanently change the value of an option add the corresponding
        command to your ``~/.pdbrc`` file. If you are programmatically using the
        Pdb instance you can also change the ``default_predicates`` class
        attribute.
        """
        ...
    
    def do_skip_hidden(self, arg): # -> None:
        """
        Change whether or not we should skip frames with the
        __tracebackhide__ attribute.
        """
        ...
    
    def do_list(self, arg): # -> None:
        """Print lines of code from the current stack frame
        """
        ...
    
    do_l = ...
    def getsourcelines(self, obj):
        ...
    
    def do_longlist(self, arg): # -> None:
        """Print lines of code from the current stack frame.

        Shows more lines than 'list' does.
        """
        ...
    
    do_ll = ...
    def do_debug(self, arg): # -> None:
        """debug code
        Enter a recursive debugger that steps through the code
        argument (which is an arbitrary expression or statement to be
        executed in the current environment).
        """
        ...
    
    def do_pdef(self, arg): # -> None:
        """Print the call signature for any callable object.

        The debugger interface to %pdef"""
        ...
    
    def do_pdoc(self, arg): # -> None:
        """Print the docstring for an object.

        The debugger interface to %pdoc."""
        ...
    
    def do_pfile(self, arg): # -> None:
        """Print (or run through pager) the file where an object is defined.

        The debugger interface to %pfile.
        """
        ...
    
    def do_pinfo(self, arg): # -> None:
        """Provide detailed information about an object.

        The debugger interface to %pinfo, i.e., obj?."""
        ...
    
    def do_pinfo2(self, arg): # -> None:
        """Provide extra detailed information about an object.

        The debugger interface to %pinfo2, i.e., obj??."""
        ...
    
    def do_psource(self, arg): # -> None:
        """Print (or run through pager) the source code for an object."""
        ...
    
    def do_where(self, arg): # -> None:
        """w(here)
        Print a stack trace, with the most recent frame at the bottom.
        An arrow indicates the "current frame", which determines the
        context of most commands. 'bt' is an alias for this command.

        Take a number as argument as an (optional) number of context line to
        print"""
        ...
    
    do_w = ...
    def break_anywhere(self, frame):
        """
        _stop_in_decorator_internals is overly restrictive, as we may still want
        to trace function calls, so we need to also update break_anywhere so
        that is we don't `stop_here`, because of debugger skip, we may still
        stop at any point inside the function

        """
        ...
    
    def stop_here(self, frame):
        ...
    
    def do_up(self, arg): # -> None:
        """u(p) [count]
        Move the current frame count (default one) levels up in the
        stack trace (to an older frame).

        Will skip hidden frames.
        """
        ...
    
    def do_down(self, arg): # -> None:
        """d(own) [count]
        Move the current frame count (default one) levels down in the
        stack trace (to a newer frame).

        Will skip hidden frames.
        """
        ...
    
    do_d = ...
    do_u = ...
    def do_context(self, context): # -> None:
        """context number_of_lines
        Set the number of lines of source code to show when displaying
        stacktrace information.
        """
        ...
    


class InterruptiblePdb(Pdb):
    """Version of debugger where KeyboardInterrupt exits the debugger altogether."""
    def cmdloop(self, intro=...): # -> None:
        """Wrap cmdloop() such that KeyboardInterrupt stops the debugger."""
        ...
    


def set_trace(frame=...): # -> None:
    """
    Start debugging from `frame`.

    If frame is not specified, debugging starts from caller's frame.
    """
    ...




from functools import partial
from typing import Any

def cubehelix(
    gamma: float = 1, s: float = 0.5, r: float = -1.5, h: float = 1
) -> dict[str, partial]: ...

gfunc: dict[int, Any] = ...

datad: dict[str, tuple[float, ...]] = ...



from ._glm import (
    PoissonRegressor as PoissonRegressor,
    GammaRegressor as GammaRegressor,
    TweedieRegressor as TweedieRegressor,
)
from ._perceptron import Perceptron as Perceptron
from ._stochastic_gradient import (
    SGDClassifier as SGDClassifier,
    SGDRegressor as SGDRegressor,
    SGDOneClassSVM as SGDOneClassSVM,
)
from ._least_angle import (
    Lars as Lars,
    LassoLars as LassoLars,
    lars_path as lars_path,
    lars_path_gram as lars_path_gram,
    LarsCV as LarsCV,
    LassoLarsCV as LassoLarsCV,
    LassoLarsIC as LassoLarsIC,
)
from ._huber import HuberRegressor as HuberRegressor
from ._base import LinearRegression as LinearRegression
from ._theil_sen import TheilSenRegressor as TheilSenRegressor
from ._omp import (
    orthogonal_mp as orthogonal_mp,
    orthogonal_mp_gram as orthogonal_mp_gram,
    OrthogonalMatchingPursuit as OrthogonalMatchingPursuit,
    OrthogonalMatchingPursuitCV as OrthogonalMatchingPursuitCV,
)
from ._logistic import (
    LogisticRegression as LogisticRegression,
    LogisticRegressionCV as LogisticRegressionCV,
)
from ._ransac import RANSACRegressor as RANSACRegressor
from ._sgd_fast import (
    Hinge as Hinge,
    Log as Log,
    ModifiedHuber as ModifiedHuber,
    SquaredLoss as SquaredLoss,
    Huber as Huber,
)
from ._bayes import BayesianRidge as BayesianRidge, ARDRegression as ARDRegression
from ._coordinate_descent import (
    Lasso as Lasso,
    ElasticNet as ElasticNet,
    LassoCV as LassoCV,
    ElasticNetCV as ElasticNetCV,
    lasso_path as lasso_path,
    enet_path as enet_path,
    MultiTaskLasso as MultiTaskLasso,
    MultiTaskElasticNet as MultiTaskElasticNet,
    MultiTaskElasticNetCV as MultiTaskElasticNetCV,
    MultiTaskLassoCV as MultiTaskLassoCV,
)
from ._passive_aggressive import (
    PassiveAggressiveClassifier as PassiveAggressiveClassifier,
    PassiveAggressiveRegressor as PassiveAggressiveRegressor,
)
from ._quantile import QuantileRegressor as QuantileRegressor
from ._ridge import (
    Ridge as Ridge,
    RidgeCV as RidgeCV,
    RidgeClassifier as RidgeClassifier,
    RidgeClassifierCV as RidgeClassifierCV,
    ridge_regression as ridge_regression,
)

__all__ = [
    "ARDRegression",
    "BayesianRidge",
    "ElasticNet",
    "ElasticNetCV",
    "Hinge",
    "Huber",
    "HuberRegressor",
    "Lars",
    "LarsCV",
    "Lasso",
    "LassoCV",
    "LassoLars",
    "LassoLarsCV",
    "LassoLarsIC",
    "LinearRegression",
    "Log",
    "LogisticRegression",
    "LogisticRegressionCV",
    "ModifiedHuber",
    "MultiTaskElasticNet",
    "MultiTaskElasticNetCV",
    "MultiTaskLasso",
    "MultiTaskLassoCV",
    "OrthogonalMatchingPursuit",
    "OrthogonalMatchingPursuitCV",
    "PassiveAggressiveClassifier",
    "PassiveAggressiveRegressor",
    "Perceptron",
    "QuantileRegressor",
    "Ridge",
    "RidgeCV",
    "RidgeClassifier",
    "RidgeClassifierCV",
    "SGDClassifier",
    "SGDRegressor",
    "SGDOneClassSVM",
    "SquaredLoss",
    "TheilSenRegressor",
    "enet_path",
    "lars_path",
    "lars_path_gram",
    "lasso_path",
    "orthogonal_mp",
    "orthogonal_mp_gram",
    "ridge_regression",
    "RANSACRegressor",
    "PoissonRegressor",
    "GammaRegressor",
    "TweedieRegressor",
]



from ..classes.reportviews import OutEdgeView, InEdgeView, DiDegreeView
from ..classes.graph import Graph
from copy import deepcopy
from functools import cached_property

from ..classes.graph import Graph
import networkx.convert as convert
from ..classes.coreviews import AdjacencyView
from ..classes.graph import Graph
from ..classes.reportviews import (
    DiDegreeView,
    InDegreeView,
    InEdgeView,
    OutDegreeView,
    OutEdgeView,
)
from ..exception import NetworkXError

__all__ = ["DiGraph"]

class _CachedPropertyResetterAdjAndSucc:
    def __set__(self, obj, value): ...

class _CachedPropertyResetterPred:
    def __set__(self, obj, value): ...

class DiGraph(Graph):

    _adj = ...  # type: ignore
    _succ = ...  # type: ignore
    _pred = ...

    def __init__(self, incoming_graph_data=None, **attr): ...
    @cached_property
    def adj(self): ...
    @cached_property
    def succ(self): ...
    @cached_property
    def pred(self): ...
    def add_node(self, node_for_adding, **attr): ...
    def add_nodes_from(self, nodes_for_adding, **attr): ...
    def remove_node(self, n): ...
    def remove_nodes_from(self, nodes): ...
    def add_edge(self, u_of_edge, v_of_edge, **attr): ...
    def add_edges_from(self, ebunch_to_add, **attr): ...
    def remove_edge(self, u, v): ...
    def remove_edges_from(self, ebunch): ...
    def has_successor(self, u, v): ...
    def has_predecessor(self, u, v): ...
    def successors(self, n): ...

    # digraph definitions
    neighbors = successors

    def predecessors(self, n): ...
    @cached_property
    def edges(self) -> OutEdgeView: ...

    # alias out_edges to edges
    @cached_property
    def out_edges(self) -> OutEdgeView: ...

    out_edges.__doc__ = ...

    @cached_property
    def in_edges(self) -> InEdgeView: ...
    @cached_property
    def degree(self) -> DiDegreeView | int: ...
    @cached_property
    def in_degree(self): ...
    @cached_property
    def out_degree(self): ...
    def clear(self): ...
    def clear_edges(self): ...
    def is_multigraph(self): ...
    def is_directed(self): ...
    def to_undirected(self, reciprocal=False, as_view=False) -> Graph: ...
    def reverse(self, copy=True): ...



from typing import Any, ClassVar
from scipy.special import expit as expit, logsumexp as logsumexp
from ..tree._tree import TREE_LEAF as TREE_LEAF
from abc import ABCMeta, abstractmethod
from numpy import ndarray
from ..dummy import DummyClassifier, DummyRegressor
from .._typing import Int, ArrayLike, MatrixLike, Float
from ..tree._tree import Tree

import numpy as np


class LossFunction(metaclass=ABCMeta):
    K: int = ...

    is_multi_class: ClassVar[bool] = ...

    def __init__(self, n_classes: Int) -> None:
        ...

    @abstractmethod
    def init_estimator(self):
        ...

    @abstractmethod
    def __call__(
        self,
        y: ArrayLike,
        raw_predictions: MatrixLike,
        sample_weight: None | ArrayLike = None,
    ):
        ...

    @abstractmethod
    def negative_gradient(self, y: ArrayLike, raw_predictions: MatrixLike, **kargs):
        ...

    def update_terminal_regions(
        self,
        tree: Tree,
        X: ArrayLike,
        y: ArrayLike,
        residual: ArrayLike,
        raw_predictions: MatrixLike,
        sample_weight: ArrayLike,
        sample_mask: ArrayLike,
        learning_rate: Float = 0.1,
        k: Int = 0,
    ) -> None:
        ...

    @abstractmethod
    def get_init_raw_predictions(self, X: ArrayLike, estimator: Any) -> ndarray:
        ...


class RegressionLossFunction(LossFunction, metaclass=ABCMeta):
    def __init__(self) -> None:
        ...

    def check_init_estimator(self, estimator: Any):
        ...

    def get_init_raw_predictions(self, X: ArrayLike, estimator: Any) -> ndarray:
        ...


class LeastSquaresError(RegressionLossFunction):
    def init_estimator(self) -> DummyRegressor:
        ...

    def __call__(
        self,
        y: ArrayLike,
        raw_predictions: MatrixLike,
        sample_weight: None | ArrayLike = None,
    ) -> Float:
        ...

    def negative_gradient(
        self, y: ArrayLike, raw_predictions: ArrayLike, **kargs
    ) -> ndarray:
        ...

    def update_terminal_regions(
        self,
        tree: Tree,
        X: ArrayLike,
        y: ArrayLike,
        residual: ArrayLike,
        raw_predictions: MatrixLike,
        sample_weight: ArrayLike,
        sample_mask: ArrayLike,
        learning_rate: Float = 0.1,
        k: Int = 0,
    ) -> None:
        ...


class LeastAbsoluteError(RegressionLossFunction):
    def init_estimator(self):
        ...

    def __call__(
        self,
        y: ArrayLike,
        raw_predictions: MatrixLike,
        sample_weight: None | ArrayLike = None,
    ):
        ...

    def negative_gradient(self, y: ArrayLike, raw_predictions: MatrixLike, **kargs):
        ...


class HuberLossFunction(RegressionLossFunction):
    def __init__(self, alpha: Float = 0.9) -> None:
        ...

    def init_estimator(self):
        ...

    def __call__(
        self,
        y: ArrayLike,
        raw_predictions: MatrixLike,
        sample_weight: None | ArrayLike = None,
    ):
        ...

    def negative_gradient(
        self,
        y: ArrayLike,
        raw_predictions: MatrixLike,
        sample_weight: None | ArrayLike = None,
        **kargs
    ):
        ...


class QuantileLossFunction(RegressionLossFunction):
    def __init__(self, alpha: Float = 0.9) -> None:
        ...

    def init_estimator(self) -> DummyRegressor:
        ...

    def __call__(
        self,
        y: ArrayLike,
        raw_predictions: MatrixLike,
        sample_weight: None | ArrayLike = None,
    ) -> Float:
        ...

    def negative_gradient(
        self, y: ArrayLike, raw_predictions: MatrixLike, **kargs
    ) -> ndarray:
        ...


class ClassificationLossFunction(LossFunction, metaclass=ABCMeta):
    def check_init_estimator(self, estimator: Any):
        ...


class BinomialDeviance(ClassificationLossFunction):
    def __init__(self, n_classes: Int) -> None:
        ...

    def init_estimator(self) -> DummyClassifier:
        ...

    def __call__(
        self,
        y: ArrayLike,
        raw_predictions: MatrixLike,
        sample_weight: None | ArrayLike = None,
    ) -> Float:
        ...

    def negative_gradient(
        self, y: ArrayLike, raw_predictions: MatrixLike, **kargs
    ) -> ndarray:
        ...

    def get_init_raw_predictions(self, X: ArrayLike, estimator: Any) -> ndarray:
        ...


class MultinomialDeviance(ClassificationLossFunction):

    is_multi_class: ClassVar[bool] = ...

    def __init__(self, n_classes: Int) -> None:
        ...

    def init_estimator(self) -> DummyClassifier:
        ...

    def __call__(
        self,
        y: ArrayLike,
        raw_predictions: MatrixLike,
        sample_weight: None | ArrayLike = None,
    ) -> Float:
        ...

    def negative_gradient(
        self, y: ArrayLike, raw_predictions: MatrixLike, k: Int = 0, **kwargs
    ) -> ndarray:
        ...

    def get_init_raw_predictions(self, X: ArrayLike, estimator: Any) -> ndarray:
        ...


class ExponentialLoss(ClassificationLossFunction):
    def __init__(self, n_classes: Int) -> None:
        ...

    def init_estimator(self):
        ...

    def __call__(
        self,
        y: ArrayLike,
        raw_predictions: MatrixLike,
        sample_weight: None | ArrayLike = None,
    ):
        ...

    def negative_gradient(self, y: ArrayLike, raw_predictions: MatrixLike, **kargs):
        ...

    def get_init_raw_predictions(self, X: ArrayLike, estimator: Any) -> ndarray:
        ...


LOSS_FUNCTIONS: dict = ...



from typing import Mapping
import warnings

from ...classes.graph import Graph

__all__ = [
    "betweenness_centrality_subset",
    "betweenness_centrality_source",
    "edge_betweenness_centrality_subset",
]

def betweenness_centrality_subset(
    G: Graph, sources, targets, normalized: bool = False, weight=None
) -> Mapping: ...
def edge_betweenness_centrality_subset(
    G: Graph, sources, targets, normalized: bool = False, weight=None
) -> Mapping: ...

# obsolete name
def betweenness_centrality_source(
    G: Graph, normalized=True, weight=None, sources=None
): ...



from typing import Mapping

from ..classes.graph import Graph

__all__ = ["constraint", "local_constraint", "effective_size"]

def mutual_weight(G: Graph, u, v, weight=None): ...
def normalized_mutual_weight(G: Graph, u, v, norm=..., weight=None): ...
def effective_size(G: Graph, nodes=None, weight: str | None = None) -> Mapping: ...
def constraint(G: Graph, nodes=None, weight: str | None = None) -> Mapping: ...
def local_constraint(G: Graph, u, v, weight: str | None = None) -> float: ...



from .blocks import (
    Block as Block,
    BoolBlock as BoolBlock,
    CategoricalBlock as CategoricalBlock,
    DatetimeBlock as DatetimeBlock,
    DatetimeTZBlock as DatetimeTZBlock,
    ExtensionBlock as ExtensionBlock,
    ObjectBlock as ObjectBlock,
    make_block as make_block,
)
from .managers import (
    BlockManager as BlockManager,
    SingleBlockManager as SingleBlockManager,
    concatenate_block_managers as concatenate_block_managers,
    create_block_manager_from_arrays as create_block_manager_from_arrays,
    create_block_manager_from_blocks as create_block_manager_from_blocks,
)



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.sparse.csgraph._matching, version: unspecified
import typing
import builtins as _mod_builtins
import numpy as _mod_numpy
import scipy.sparse.csr as _mod_scipy_sparse_csr

BTYPE = _mod_numpy.uint8
DTYPE = _mod_numpy.float64
ITYPE = _mod_numpy.int32
__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
csr_matrix = _mod_scipy_sparse_csr.csr_matrix
def isspmatrix_coo(x) -> typing.Any:
    'Is x of coo_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a coo matrix\n\n    Returns\n    -------\n    bool\n        True if x is a coo matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import coo_matrix, isspmatrix_coo\n    >>> isspmatrix_coo(coo_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import coo_matrix, csr_matrix, isspmatrix_coo\n    >>> isspmatrix_coo(csr_matrix([[5]]))\n    False\n    '
    ...

def isspmatrix_csc(x) -> typing.Any:
    'Is x of csc_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a csc matrix\n\n    Returns\n    -------\n    bool\n        True if x is a csc matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import csc_matrix, isspmatrix_csc\n    >>> isspmatrix_csc(csc_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import csc_matrix, csr_matrix, isspmatrix_csc\n    >>> isspmatrix_csc(csr_matrix([[5]]))\n    False\n    '
    ...

def isspmatrix_csr(x) -> typing.Any:
    'Is x of csr_matrix type?\n\n    Parameters\n    ----------\n    x\n        object to check for being a csr matrix\n\n    Returns\n    -------\n    bool\n        True if x is a csr matrix, False otherwise\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix, isspmatrix_csr\n    >>> isspmatrix_csr(csr_matrix([[5]]))\n    True\n\n    >>> from scipy.sparse import csc_matrix, csr_matrix, isspmatrix_csc\n    >>> isspmatrix_csr(csc_matrix([[5]]))\n    False\n    '
    ...

def maximum_bipartite_matching(graph, perm_type=...) -> typing.Any:
    '\n    maximum_bipartite_matching(graph, perm_type=\'row\')\n\n    Returns a matching of a bipartite graph whose cardinality is as least that\n    of any given matching of the graph.\n\n    Parameters\n    ----------\n    graph : sparse matrix\n        Input sparse in CSR format whose rows represent one partition of the\n        graph and whose columns represent the other partition. An edge between\n        two vertices is indicated by the corresponding entry in the matrix\n        existing in its sparse representation.\n    perm_type : str, {\'row\', \'column\'}\n        Which partition to return the matching in terms of: If ``\'row\'``, the\n        function produces an array whose length is the number of columns in the\n        input, and whose :math:`j`\'th element is the row matched to the\n        :math:`j`\'th column. Conversely, if ``perm_type`` is ``\'column\'``, this\n        returns the columns matched to each row.\n\n    Returns\n    -------\n    perm : ndarray\n        A matching of the vertices in one of the two partitions. Unmatched\n        vertices are represented by a ``-1`` in the result.\n\n    Notes\n    -----\n    This function implements the Hopcroft--Karp algorithm [1]_. Its time\n    complexity is :math:`O(\\lvert E \\rvert \\sqrt{\\lvert V \\rvert})`, and its\n    space complexity is linear in the number of rows. In practice, this\n    asymmetry between rows and columns means that it can be more efficient to\n    transpose the input if it contains more columns than rows.\n\n    By Konig\'s theorem, the cardinality of the matching is also the number of\n    vertices appearing in a minimum vertex cover of the graph.\n\n    Note that if the sparse representation contains explicit zeros, these are\n    still counted as edges.\n\n    The implementation was changed in SciPy 1.4.0 to allow matching of general\n    bipartite graphs, where previous versions would assume that a perfect\n    matching existed. As such, code written against 1.4.0 will not necessarily\n    work on older versions.\n\n    References\n    ----------\n    .. [1] John E. Hopcroft and Richard M. Karp. "An n^{5 / 2} Algorithm for\n           Maximum Matchings in Bipartite Graphs" In: SIAM Journal of Computing\n           2.4 (1973), pp. 225--231. :doi:`10.1137/0202019`\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import maximum_bipartite_matching\n\n    As a simple example, consider a bipartite graph in which the partitions\n    contain 2 and 3 elements respectively. Suppose that one partition contains\n    vertices labelled 0 and 1, and that the other partition contains vertices\n    labelled A, B, and C. Suppose that there are edges connecting 0 and C,\n    1 and A, and 1 and B. This graph would then be represented by the following\n    sparse matrix:\n\n    >>> graph = csr_matrix([[0, 0, 1], [1, 1, 0]])\n\n    Here, the 1s could be anything, as long as they end up being stored as\n    elements in the sparse matrix. We can now calculate maximum matchings as\n    follows:\n\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'column\'))\n    [2 0]\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'row\'))\n    [ 1 -1  0]\n\n    The first output tells us that 1 and 2 are matched with C and A\n    respectively, and the second output tells us that A, B, and C are matched\n    with 1, nothing, and 0 respectively.\n\n    Note that explicit zeros are still converted to edges. This means that a\n    different way to represent the above graph is by using the CSR structure\n    directly as follows:\n\n    >>> data = [0, 0, 0]\n    >>> indices = [2, 0, 1]\n    >>> indptr = [0, 1, 3]\n    >>> graph = csr_matrix((data, indices, indptr))\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'column\'))\n    [2 0]\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'row\'))\n    [ 1 -1  0]\n\n    When one or both of the partitions are empty, the matching is empty as\n    well:\n\n    >>> graph = csr_matrix((2, 0))\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'column\'))\n    [-1 -1]\n    >>> print(maximum_bipartite_matching(graph, perm_type=\'row\'))\n    []\n\n    When the input matrix is square, and the graph is known to admit a perfect\n    matching, i.e. a matching with the property that every vertex in the graph\n    belongs to some edge in the matching, then one can view the output as the\n    permutation of rows (or columns) turning the input matrix into one with the\n    property that all diagonal elements are non-empty:\n\n    >>> a = [[0, 1, 2, 0], [1, 0, 0, 1], [2, 0, 0, 3], [0, 1, 3, 0]]\n    >>> graph = csr_matrix(a)\n    >>> perm = maximum_bipartite_matching(graph, perm_type=\'row\')\n    >>> print(graph[perm].toarray())\n    [[1 0 0 1]\n     [0 1 2 0]\n     [0 1 3 0]\n     [2 0 0 3]]\n\n    '
    ...

def min_weight_full_bipartite_matching(biadjacency_matrix, maximize=...) -> typing.Any:
    '\n    min_weight_full_bipartite_matching(biadjacency_matrix, maximize=False)\n\n    Returns the minimum weight full matching of a bipartite graph.\n\n    .. versionadded:: 1.6.0\n\n    Parameters\n    ----------\n    biadjacency_matrix : sparse matrix\n        Biadjacency matrix of the bipartite graph: A sparse matrix in CSR, CSC,\n        or COO format whose rows represent one partition of the graph and whose\n        columns represent the other partition. An edge between two vertices is\n        indicated by the corresponding entry in the matrix, and the weight of\n        the edge is given by the value of that entry. This should not be\n        confused with the full adjacency matrix of the graph, as we only need\n        the submatrix defining the bipartite structure.\n\n    maximize : bool (default: False)\n        Calculates a maximum weight matching if true.\n\n    Returns\n    -------\n    row_ind, col_ind : array\n        An array of row indices and one of corresponding column indices giving\n        the optimal matching. The total weight of the matching can be computed\n        as ``graph[row_ind, col_ind].sum()``. The row indices will be\n        sorted; in the case of a square matrix they will be equal to\n        ``numpy.arange(graph.shape[0])``.\n\n    Notes\n    -----\n\n    Let :math:`G = ((U, V), E)` be a weighted bipartite graph with non-zero\n    weights :math:`w : E \\to \\mathbb{R} \\setminus \\{0\\}`. This function then\n    produces a matching :math:`M \\subseteq E` with cardinality\n\n    .. math::\n       \\lvert M \\rvert = \\min(\\lvert U \\rvert, \\lvert V \\rvert),\n\n    which minimizes the sum of the weights of the edges included in the\n    matching, :math:`\\sum_{e \\in M} w(e)`, or raises an error if no such\n    matching exists.\n\n    When :math:`\\lvert U \\rvert = \\lvert V \\rvert`, this is commonly\n    referred to as a perfect matching; here, since we allow\n    :math:`\\lvert U \\rvert` and :math:`\\lvert V \\rvert` to differ, we\n    follow Karp [1]_ and refer to the matching as *full*.\n\n    This function implements the LAPJVsp algorithm [2]_, short for "Linear\n    assignment problem, Jonker--Volgenant, sparse".\n\n    The problem it solves is equivalent to the rectangular linear assignment\n    problem. [3]_ As such, this function can be used to solve the same problems\n    as :func:`scipy.optimize.linear_sum_assignment`. That function may perform\n    better when the input is dense, or for certain particular types of inputs,\n    such as those for which the :math:`(i, j)`\'th entry is the distance between\n    two points in Euclidean space.\n\n    If no full matching exists, this function raises a ``ValueError``. For\n    determining the size of the largest matching in the graph, see\n    :func:`maximum_bipartite_matching`.\n\n    We require that weights are non-zero only to avoid issues with the handling\n    of explicit zeros when converting between different sparse representations.\n    Zero weights can be handled by adding a constant to all weights, so that\n    the resulting matrix contains no zeros.\n\n    References\n    ----------\n    .. [1] Richard Manning Karp:\n       An algorithm to Solve the m x n Assignment Problem in Expected Time\n       O(mn log n).\n       Networks, 10(2):143-152, 1980.\n    .. [2] Roy Jonker and Anton Volgenant:\n       A Shortest Augmenting Path Algorithm for Dense and Sparse Linear\n       Assignment Problems.\n       Computing 38:325-340, 1987.\n    .. [3] https://en.wikipedia.org/wiki/Assignment_problem\n\n    Examples\n    --------\n    >>> from scipy.sparse import csr_matrix\n    >>> from scipy.sparse.csgraph import min_weight_full_bipartite_matching\n\n    Let us first consider an example in which all weights are equal:\n\n    >>> biadjacency_matrix = csr_matrix([[1, 1, 1], [1, 0, 0], [0, 1, 0]])\n\n    Here, all we get is a perfect matching of the graph:\n\n    >>> print(min_weight_full_bipartite_matching(biadjacency_matrix)[1])\n    [2 0 1]\n\n    That is, the first, second, and third rows are matched with the third,\n    first, and second column respectively. Note that in this example, the 0\n    in the input matrix does *not* correspond to an edge with weight 0, but\n    rather a pair of vertices not paired by an edge.\n\n    Note also that in this case, the output matches the result of applying\n    :func:`maximum_bipartite_matching`:\n\n    >>> from scipy.sparse.csgraph import maximum_bipartite_matching\n    >>> biadjacency = csr_matrix([[1, 1, 1], [1, 0, 0], [0, 1, 0]])\n    >>> print(maximum_bipartite_matching(biadjacency, perm_type=\'column\'))\n    [2 0 1]\n\n    When multiple edges are available, the ones with lowest weights are\n    preferred:\n\n    >>> biadjacency = csr_matrix([[3, 3, 6], [4, 3, 5], [10, 1, 8]])\n    >>> row_ind, col_ind = min_weight_full_bipartite_matching(biadjacency)\n    >>> print(col_ind)\n    [0 2 1]\n\n    The total weight in this case is :math:`3 + 5 + 1 = 9`:\n\n    >>> print(biadjacency[row_ind, col_ind].sum())\n    9\n\n    When the matrix is not square, i.e. when the two partitions have different\n    cardinalities, the matching is as large as the smaller of the two\n    partitions:\n\n    >>> biadjacency = csr_matrix([[0, 1, 1], [0, 2, 3]])\n    >>> row_ind, col_ind = min_weight_full_bipartite_matching(biadjacency)\n    >>> print(row_ind, col_ind)\n    [0 1] [2 1]\n    >>> biadjacency = csr_matrix([[0, 1], [3, 1], [1, 4]])\n    >>> row_ind, col_ind = min_weight_full_bipartite_matching(biadjacency)\n    >>> print(row_ind, col_ind)\n    [0 2] [1 0]\n\n    When one or both of the partitions are empty, the matching is empty as\n    well:\n\n    >>> biadjacency = csr_matrix((2, 0))\n    >>> row_ind, col_ind = min_weight_full_bipartite_matching(biadjacency)\n    >>> print(row_ind, col_ind)\n    [] []\n\n    In general, we will always reach the same sum of weights as if we had used\n    :func:`scipy.optimize.linear_sum_assignment` but note that for that one,\n    missing edges are represented by a matrix entry of ``float(\'inf\')``. Let us\n    generate a random sparse matrix with integer entries between 1 and 10:\n\n    >>> import numpy as np\n    >>> from scipy.sparse import random\n    >>> from scipy.optimize import linear_sum_assignment\n    >>> sparse = random(10, 10, random_state=42, density=.5, format=\'coo\') * 10\n    >>> sparse.data = np.ceil(sparse.data)\n    >>> dense = sparse.toarray()\n    >>> dense = np.full(sparse.shape, np.inf)\n    >>> dense[sparse.row, sparse.col] = sparse.data\n    >>> sparse = sparse.tocsr()\n    >>> row_ind, col_ind = linear_sum_assignment(dense)\n    >>> print(dense[row_ind, col_ind].sum())\n    28.0\n    >>> row_ind, col_ind = min_weight_full_bipartite_matching(sparse)\n    >>> print(sparse[row_ind, col_ind].sum())\n    28.0\n\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




from ...classes.graph import Graph
from ...utils.decorators import not_implemented_for

__all__ = [
    "number_strongly_connected_components",
    "strongly_connected_components",
    "is_strongly_connected",
    "strongly_connected_components_recursive",
    "kosaraju_strongly_connected_components",
    "condensation",
]

def strongly_connected_components(G: Graph): ...
def kosaraju_strongly_connected_components(G: Graph, source=None): ...
def strongly_connected_components_recursive(G: Graph): ...
def number_strongly_connected_components(G: Graph): ...
def is_strongly_connected(G: Graph) -> bool: ...
def condensation(G: Graph, scc=None): ...



from functools import reduce
import numpy as np
from ..draw import polygon
from .._shared.version_requirements import require

LEFT_CLICK: int = ...
RIGHT_CLICK: int = ...

def _mask_from_vertices(vertices, shape, label): ...
@require("matplotlib", ">=3.0.3")
def _draw_polygon(ax, vertices, alpha=0.4): ...
@require("matplotlib", ">=3.0.3")
def manual_polygon_segmentation(
    image, alpha: float = 0.4, return_all: bool = False
): ...
@require("matplotlib", ">=3.0.3")
def manual_lasso_segmentation(image, alpha: float = 0.4, return_all: bool = False): ...



import numpy as np
from matplotlib._typing import *
from .triinterpolate import TriInterpolator
from .triangulation import Triangulation

class TriRefiner:
    def __init__(self, triangulation: Triangulation) -> None: ...

class UniformTriRefiner(TriRefiner):
    def __init__(self, triangulation: Triangulation) -> None: ...
    def refine_triangulation(
        self, return_tri_index: bool = False, subdiv: int = 3
    ) -> tuple[Triangulation, np.ndarray]: ...
    def refine_field(
        self,
        z: ArrayLike,
        triinterpolator: TriInterpolator = ...,
        subdiv: int = 3,
    ) -> tuple[Triangulation, np.ndarray]: ...



from ..classes.graph import Graph
from ..utils import py_random_state

__all__ = [
    "powerlaw_sequence",
    "zipf_rv",
    "cumulative_distribution",
    "discrete_sequence",
    "random_weighted_sample",
    "weighted_choice",
]

# The same helpers for choosing random sequences from distributions
# uses Python's random module
# https://docs.python.org/3/library/random.html

@py_random_state(2)
def powerlaw_sequence(n, exponent=2.0, seed=None): ...
@py_random_state(2)
def zipf_rv(alpha: float, xmin: int = 1, seed=None) -> int: ...
def cumulative_distribution(distribution): ...
@py_random_state(3)
def discrete_sequence(n, distribution=None, cdistribution=None, seed=None): ...
@py_random_state(2)
def random_weighted_sample(mapping, k, seed=None): ...
@py_random_state(1)
def weighted_choice(mapping, seed=None): ...



from typing import (
    List,
    Sequence,
    Union,
)

from pandas.core.base import PandasObject
from pandas.core.indexes.api import Index
from pandas.core.internals.blocks import Block

class BlockManager(PandasObject):
    axes = ...
    blocks = ...
    def __init__(
        self,
        blocks: Sequence[Block],
        axes: Sequence[Index],
        do_integrity_check: bool = ...,
    ) -> None: ...
    def make_empty(self, axes=...): ...
    def __nonzero__(self): ...
    __bool__ = ...
    @property
    def shape(self): ...
    @property
    def ndim(self) -> int: ...
    def set_axis(self, axis, new_labels) -> None: ...
    def rename_axis(self, mapper, axis, copy: bool = ..., level=...): ...
    @property
    def items(self): ...
    def get_dtype_counts(self): ...
    def get_dtypes(self): ...
    def __len__(self) -> int: ...
    def reduce(self, func, *args, **kwargs): ...
    def apply(self, f, filter=..., **kwargs): ...
    def quantile(
        self,
        axis: int = ...,
        consolidate: bool = ...,
        transposed: bool = ...,
        interpolation: str = ...,
        qs=...,
        numeric_only=...,
    ): ...
    def isna(self, func): ...
    def where(self, **kwargs): ...
    def setitem(self, **kwargs): ...
    def putmask(self, **kwargs): ...
    def diff(self, **kwargs): ...
    def interpolate(self, **kwargs): ...
    def shift(self, **kwargs): ...
    def fillna(self, **kwargs): ...
    def downcast(self, **kwargs): ...
    def astype(self, dtype, copy: bool = ..., errors: str = ...): ...
    def convert(self, **kwargs): ...
    def replace(self, value, **kwargs): ...
    def replace_list(
        self, src_list, dest_list, inplace: bool = ..., regex: bool = ...
    ): ...
    def is_consolidated(self): ...
    @property
    def is_mixed_type(self): ...
    @property
    def is_numeric_mixed_type(self): ...
    @property
    def is_datelike_mixed_type(self): ...
    @property
    def any_extension_types(self): ...
    @property
    def is_view(self): ...
    def get_bool_data(self, copy: bool = ...): ...
    def get_numeric_data(self, copy: bool = ...): ...
    def combine(self, blocks, copy: bool = ...): ...
    def get_slice(self, slobj: slice, axis: int = ...): ...
    def __contains__(self, item) -> bool: ...
    @property
    def nblocks(self) -> int: ...
    def copy(self, deep: bool = ...): ...
    def as_array(self, transpose: bool = ..., items=...): ...
    def to_dict(self, copy: bool = ...): ...
    def fast_xs(self, loc): ...
    def consolidate(self): ...
    def get(self, item): ...
    def iget(self, i): ...
    def delete(self, item) -> None: ...
    def set(self, item, value): ...
    def insert(self, loc: int, item, value, allow_duplicates: bool = ...): ...
    def reindex_axis(
        self, new_index, axis, method=..., limit=..., fill_value=..., copy: bool = ...
    ): ...
    def reindex_indexer(
        self,
        new_axis,
        indexer,
        axis,
        fill_value=...,
        allow_dups: bool = ...,
        copy: bool = ...,
    ): ...
    def take(
        self, indexer, axis: int = ..., verify: bool = ..., convert: bool = ...
    ): ...
    def equals(self, other): ...
    def unstack(self, unstacker_func, fill_value): ...

class SingleBlockManager(BlockManager):
    ndim: int = ...
    axes = ...
    blocks = ...
    def __init__(
        self,
        block: Block,
        axis: Union[Index, List[Index]],
        do_integrity_check: bool = ...,
        fastpath: bool = ...,
    ) -> None: ...
    def get_slice(self, slobj, axis: int = ...): ...
    @property
    def index(self): ...
    @property
    def dtype(self): ...
    @property
    def array_dtype(self): ...
    def get_dtype_counts(self): ...
    def get_dtypes(self): ...
    def external_values(self): ...
    def internal_values(self): ...
    def get_values(self): ...
    def is_consolidated(self): ...
    def delete(self, item) -> None: ...
    def fast_xs(self, loc): ...
    def concat(self, to_concat, new_axis): ...

def create_block_manager_from_blocks(blocks, axes): ...
def create_block_manager_from_arrays(arrays, names, axes): ...
def construction_error(tot_items, block_shape, axes, e=...) -> None: ...
def form_blocks(arrays, names, axes): ...
def concatenate_block_managers(mgrs_indexers, axes, concat_axis, copy): ...



from scipy import ndimage

from .._shared.utils import deprecate_kwarg

def _label_bool(image, background=None, return_num=False, connectivity=None): ...
@deprecate_kwarg(
    {"input": "label_image"}, deprecated_version="0.19", removed_version="1.0"
)
def label(
    label_image,
    background: int | None = None,
    return_num: bool = False,
    connectivity: int | None = None,
): ...




import numpy as np


def lloyd_iter_chunked_dense(
        X: np.ndarray,
        sample_weight: np.ndarray,
        centers_old: np.ndarray,
        centers_new: np.ndarray,
        weight_in_clusters: np.ndarray,
        labels: np.ndarray,
        center_shift: np.ndarray,
        n_threads: int,
        update_centers: bool=True) -> None:
    """Single iteration of K-means lloyd algorithm with dense input.
    Update labels and centers (inplace), for one iteration, distributed
    over data chunks.
    Parameters
    ----------
    X : ndarray of shape (n_samples, n_features), dtype=floating
        The observations to cluster.
    sample_weight : ndarray of shape (n_samples,), dtype=floating
        The weights for each observation in X.
    centers_old : ndarray of shape (n_clusters, n_features), dtype=floating
        Centers before previous iteration, placeholder for the centers after
        previous iteration.
    centers_new : ndarray of shape (n_clusters, n_features), dtype=floating
        Centers after previous iteration, placeholder for the new centers
        computed during this iteration. `centers_new` can be `None` if
        `update_centers` is False.
    weight_in_clusters : ndarray of shape (n_clusters,), dtype=floating
        Placeholder for the sums of the weights of every observation assigned
        to each center. `weight_in_clusters` can be `None` if `update_centers`
        is False.
    labels : ndarray of shape (n_samples,), dtype=int
        labels assignment.
    center_shift : ndarray of shape (n_clusters,), dtype=floating
        Distance between old and new centers.
    n_threads : int
        The number of threads to be used by openmp.
    update_centers : bool
        - If True, the labels and the new centers will be computed, i.e. runs
          the E-step and the M-step of the algorithm.
        - If False, only the labels will be computed, i.e runs the E-step of
          the algorithm. This is useful especially when calling predict on a
          fitted model.
    """
    ...


def lloyd_iter_chunked_sparse(
        X: np.ndarray,
        sample_weight: np.ndarray,
        centers_old: np.ndarray,
        centers_new: np.ndarray,
        weight_in_clusters: np.ndarray,
        labels: np.ndarray,
        center_shift: np.ndarray,
        n_threads: int,
        update_centers: bool=True) -> None:
    """Single iteration of K-means lloyd algorithm with sparse input.
    Update labels and centers (inplace), for one iteration, distributed
    over data chunks.
    Parameters
    ----------
    X : sparse matrix of shape (n_samples, n_features), dtype=floating
        The observations to cluster. Must be in CSR format.
    sample_weight : ndarray of shape (n_samples,), dtype=floating
        The weights for each observation in X.
    centers_old : ndarray of shape (n_clusters, n_features), dtype=floating
        Centers before previous iteration, placeholder for the centers after
        previous iteration.
    centers_new : ndarray of shape (n_clusters, n_features), dtype=floating
        Centers after previous iteration, placeholder for the new centers
        computed during this iteration. `centers_new` can be `None` if
        `update_centers` is False.
    weight_in_clusters : ndarray of shape (n_clusters,), dtype=floating
        Placeholder for the sums of the weights of every observation assigned
        to each center. `weight_in_clusters` can be `None` if `update_centers`
        is False.
    labels : ndarray of shape (n_samples,), dtype=int
        labels assignment.
    center_shift : ndarray of shape (n_clusters,), dtype=floating
        Distance between old and new centers.
    n_threads : int
        The number of threads to be used by openmp.
    update_centers : bool
        - If True, the labels and the new centers will be computed, i.e. runs
          the E-step and the M-step of the algorithm.
        - If False, only the labels will be computed, i.e runs the E-step of
          the algorithm. This is useful especially when calling predict on a
          fitted model.
    """
    ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.optimize._highs._mpswriter, version: unspecified
import typing
import builtins as _mod_builtins
import scipy.sparse.csc as _mod_scipy_sparse_csc

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
csc_matrix = _mod_scipy_sparse_csc.csc_matrix
def mpswriter() -> typing.Any:
    'MPS writer: create MPS file from matrices.\n\n    Parameters\n    ----------\n    filename : bytes (string)\n        Name of MPS file to write out to.  Will be overwritten.\n    c : 1-D array (numcol,)\n        Objective coefficient values (assumes minimization).\n    A : 2-D array (numrow, numcol), scipy.sparse.csc_matrix\n        Sparse inequality constraint matrix.\n    lhs : 1-D array (numrow,)\n        Left hand side inequality values.\n    rhs : 1-D array (numrow,)\n        Right hand side inequality values.\n    lb : 1-D array (numcol,)\n        Lower bounds of solution variables.\n    ub : 1-D array (numcol,)\n        Upper bounds of solution variables.\n    integer_valued : 1-D array (numint,)\n        Indices of integer valued solution variables.\n    use_free_format : bool, optional\n        Use MPS free format.  Default is False.\n\n    Notes\n    -----\n    Wrapper over HiGHS `writeMPS` function.\n    '
    ...

def __getattr__(name) -> typing.Any:
    ...




from .assortativity import *
from .asteroidal import is_at_free as is_at_free, find_asteroidal_triple as find_asteroidal_triple
from .boundary import edge_boundary as edge_boundary, node_boundary as node_boundary
from .bridges import bridges as bridges, has_bridges as has_bridges, local_bridges as local_bridges
from .chains import chain_decomposition as chain_decomposition
from .centrality import *
from .chordal import (
    is_chordal as is_chordal,
    find_induced_nodes as find_induced_nodes,
    chordal_graph_cliques as chordal_graph_cliques,
    chordal_graph_treewidth as chordal_graph_treewidth,
    NetworkXTreewidthBoundExceeded as NetworkXTreewidthBoundExceeded,
    complete_to_chordal_graph as complete_to_chordal_graph,
)
from .cluster import (
    triangles as triangles,
    average_clustering as average_clustering,
    clustering as clustering,
    transitivity as transitivity,
    square_clustering as square_clustering,
    generalized_degree as generalized_degree,
)
from .clique import (
    find_cliques as find_cliques,
    find_cliques_recursive as find_cliques_recursive,
    make_max_clique_graph as make_max_clique_graph,
    make_clique_bipartite as make_clique_bipartite,
    graph_clique_number as graph_clique_number,
    graph_number_of_cliques as graph_number_of_cliques,
    node_clique_number as node_clique_number,
    number_of_cliques as number_of_cliques,
    cliques_containing_node as cliques_containing_node,
    enumerate_all_cliques as enumerate_all_cliques,
    max_weight_clique as max_weight_clique,
)
from .communicability_alg import communicability as communicability, communicability_exp as communicability_exp
from .components import *
from .coloring import *
from .core import (
    core_number as core_number,
    find_cores as find_cores,
    k_core as k_core,
    k_shell as k_shell,
    k_crust as k_crust,
    k_corona as k_corona,
    k_truss as k_truss,
    onion_layers as onion_layers,
)
from .covering import min_edge_cover as min_edge_cover, is_edge_cover as is_edge_cover
from .cycles import (
    cycle_basis as cycle_basis,
    simple_cycles as simple_cycles,
    recursive_simple_cycles as recursive_simple_cycles,
    find_cycle as find_cycle,
    minimum_cycle_basis as minimum_cycle_basis,
)
from .cuts import (
    boundary_expansion as boundary_expansion,
    conductance as conductance,
    cut_size as cut_size,
    edge_expansion as edge_expansion,
    mixing_expansion as mixing_expansion,
    node_expansion as node_expansion,
    normalized_cut_size as normalized_cut_size,
    volume as volume,
)
from .d_separation import d_separated as d_separated
from .dag import (
    descendants as descendants,
    ancestors as ancestors,
    topological_sort as topological_sort,
    lexicographical_topological_sort as lexicographical_topological_sort,
    all_topological_sorts as all_topological_sorts,
    topological_generations as topological_generations,
    is_directed_acyclic_graph as is_directed_acyclic_graph,
    is_aperiodic as is_aperiodic,
    transitive_closure as transitive_closure,
    transitive_closure_dag as transitive_closure_dag,
    transitive_reduction as transitive_reduction,
    antichains as antichains,
    dag_longest_path as dag_longest_path,
    dag_longest_path_length as dag_longest_path_length,
    dag_to_branching as dag_to_branching,
)
from .distance_measures import (
    extrema_bounding as extrema_bounding,
    eccentricity as eccentricity,
    diameter as diameter,
    radius as radius,
    periphery as periphery,
    center as center,
    barycenter as barycenter,
    resistance_distance as resistance_distance,
)
from .distance_regular import (
    is_distance_regular as is_distance_regular,
    is_strongly_regular as is_strongly_regular,
    intersection_array as intersection_array,
    global_parameters as global_parameters,
)
from .dominance import immediate_dominators as immediate_dominators, dominance_frontiers as dominance_frontiers
from .dominating import dominating_set as dominating_set, is_dominating_set as is_dominating_set
from .efficiency_measures import (
    efficiency as efficiency,
    local_efficiency as local_efficiency,
    global_efficiency as global_efficiency,
)
from .euler import (
    is_eulerian as is_eulerian,
    eulerian_circuit as eulerian_circuit,
    eulerize as eulerize,
    is_semieulerian as is_semieulerian,
    has_eulerian_path as has_eulerian_path,
    eulerian_path as eulerian_path,
)
from .graphical import (
    is_graphical as is_graphical,
    is_multigraphical as is_multigraphical,
    is_pseudographical as is_pseudographical,
    is_digraphical as is_digraphical,
    is_valid_degree_sequence_erdos_gallai as is_valid_degree_sequence_erdos_gallai,
    is_valid_degree_sequence_havel_hakimi as is_valid_degree_sequence_havel_hakimi,
)
from .hierarchy import flow_hierarchy as flow_hierarchy
from .hybrid import kl_connected_subgraph as kl_connected_subgraph, is_kl_connected as is_kl_connected
from .link_analysis import *
from .link_prediction import (
    resource_allocation_index as resource_allocation_index,
    jaccard_coefficient as jaccard_coefficient,
    adamic_adar_index as adamic_adar_index,
    preferential_attachment as preferential_attachment,
    cn_soundarajan_hopcroft as cn_soundarajan_hopcroft,
    ra_index_soundarajan_hopcroft as ra_index_soundarajan_hopcroft,
    within_inter_cluster as within_inter_cluster,
    common_neighbor_centrality as common_neighbor_centrality,
)
from .lowest_common_ancestors import (
    all_pairs_lowest_common_ancestor as all_pairs_lowest_common_ancestor,
    tree_all_pairs_lowest_common_ancestor as tree_all_pairs_lowest_common_ancestor,
    lowest_common_ancestor as lowest_common_ancestor,
)
from .isolate import is_isolate as is_isolate, isolates as isolates, number_of_isolates as number_of_isolates
from .matching import (
    is_matching as is_matching,
    is_maximal_matching as is_maximal_matching,
    is_perfect_matching as is_perfect_matching,
    max_weight_matching as max_weight_matching,
    min_weight_matching as min_weight_matching,
    maximal_matching as maximal_matching,
)
from .minors import *
from .mis import maximal_independent_set as maximal_independent_set
from .moral import moral_graph as moral_graph
from .non_randomness import non_randomness as non_randomness
from .operators import *
from .planarity import check_planarity as check_planarity, is_planar as is_planar, PlanarEmbedding as PlanarEmbedding
from .planar_drawing import (
    combinatorial_embedding_to_pos as combinatorial_embedding_to_pos,
)
from .reciprocity import reciprocity as reciprocity, overall_reciprocity as overall_reciprocity
from .regular import is_regular as is_regular, is_k_regular as is_k_regular, k_factor as k_factor
from .richclub import rich_club_coefficient as rich_club_coefficient
from .shortest_paths import *
from .similarity import (
    graph_edit_distance as graph_edit_distance,
    optimal_edit_paths as optimal_edit_paths,
    optimize_graph_edit_distance as optimize_graph_edit_distance,
    optimize_edit_paths as optimize_edit_paths,
    simrank_similarity as simrank_similarity,
    simrank_similarity_numpy as simrank_similarity_numpy,
    panther_similarity as panther_similarity,
    generate_random_paths as generate_random_paths,
)
from .graph_hashing import (
    weisfeiler_lehman_graph_hash as weisfeiler_lehman_graph_hash,
    weisfeiler_lehman_subgraph_hashes as weisfeiler_lehman_subgraph_hashes,
)
from .simple_paths import (
    all_simple_paths as all_simple_paths,
    is_simple_path as is_simple_path,
    shortest_simple_paths as shortest_simple_paths,
    all_simple_edge_paths as all_simple_edge_paths,
)
from .smallworld import (
    random_reference as random_reference,
    lattice_reference as lattice_reference,
    sigma as sigma,
    omega as omega,
)
from .smetric import s_metric as s_metric
from .structuralholes import constraint as constraint, local_constraint as local_constraint, effective_size as effective_size
from .sparsifiers import spanner as spanner
from .summarization import dedensify as dedensify, snap_aggregation as snap_aggregation
from .swap import double_edge_swap as double_edge_swap, connected_double_edge_swap as connected_double_edge_swap
from .traversal import *
from .triads import (
    triadic_census as triadic_census,
    is_triad as is_triad,
    all_triplets as all_triplets,
    all_triads as all_triads,
    triads_by_type as triads_by_type,
    triad_type as triad_type,
    random_triad as random_triad,
)
from .vitality import closeness_vitality as closeness_vitality
from .voronoi import voronoi_cells as voronoi_cells
from .wiener import wiener_index as wiener_index
from .polynomials import tutte_polynomial as tutte_polynomial, chromatic_polynomial as chromatic_polynomial

# Make certain subpackages available to the user as direct imports from
# the `networkx` namespace.
from . import approximation as approximation
from . import assortativity as assortativity
from . import bipartite as bipartite
from . import node_classification as node_classification
from . import centrality as centrality
from . import chordal as chordal
from . import cluster as cluster
from . import clique as clique
from . import components as components
from . import connectivity as connectivity
from . import community as community
from . import coloring as coloring
from . import flow as flow
from . import isomorphism as isomorphism
from . import link_analysis as link_analysis
from . import lowest_common_ancestors as lowest_common_ancestors
from . import operators as operators
from . import shortest_paths as shortest_paths
from . import tournament as tournament
from . import traversal as traversal
from . import tree as tree

# Make certain functions from some of the previous subpackages available
# to the user as direct imports from the `networkx` namespace.
from .bipartite import (
    complete_bipartite_graph as complete_bipartite_graph,
)
from .bipartite import is_bipartite as is_bipartite
from .bipartite import project as project
from .bipartite import projected_graph as projected_graph
from .connectivity import (
    all_pairs_node_connectivity as all_pairs_node_connectivity,
)
from .connectivity import all_node_cuts as all_node_cuts
from .connectivity import (
    average_node_connectivity as average_node_connectivity,
)
from .connectivity import edge_connectivity as edge_connectivity
from .connectivity import edge_disjoint_paths as edge_disjoint_paths
from .connectivity import k_components as k_components
from .connectivity import k_edge_components as k_edge_components
from .connectivity import k_edge_subgraphs as k_edge_subgraphs
from .connectivity import k_edge_augmentation as k_edge_augmentation
from .connectivity import is_k_edge_connected as is_k_edge_connected
from .connectivity import minimum_edge_cut as minimum_edge_cut
from .connectivity import minimum_node_cut as minimum_node_cut
from .connectivity import node_connectivity as node_connectivity
from .connectivity import node_disjoint_paths as node_disjoint_paths
from .connectivity import stoer_wagner as stoer_wagner
from .flow import capacity_scaling as capacity_scaling
from .flow import cost_of_flow as cost_of_flow
from .flow import gomory_hu_tree as gomory_hu_tree
from .flow import max_flow_min_cost as max_flow_min_cost
from .flow import maximum_flow as maximum_flow
from .flow import maximum_flow_value as maximum_flow_value
from .flow import min_cost_flow as min_cost_flow
from .flow import min_cost_flow_cost as min_cost_flow_cost
from .flow import minimum_cut as minimum_cut
from .flow import minimum_cut_value as minimum_cut_value
from .flow import network_simplex as network_simplex
from .isomorphism import could_be_isomorphic as could_be_isomorphic
from .isomorphism import (
    fast_could_be_isomorphic as fast_could_be_isomorphic,
)
from .isomorphism import (
    faster_could_be_isomorphic as faster_could_be_isomorphic,
)
from .isomorphism import is_isomorphic as is_isomorphic
from .tree.branchings import maximum_branching as maximum_branching
from .tree.branchings import (
    maximum_spanning_arborescence as maximum_spanning_arborescence,
)
from .tree.branchings import minimum_branching as minimum_branching
from .tree.branchings import (
    minimum_spanning_arborescence as minimum_spanning_arborescence,
)
from .tree.branchings import (
    ArborescenceIterator as ArborescenceIterator,
)
from .tree.coding import (
    from_nested_tuple as from_nested_tuple,
    from_prufer_sequence as from_prufer_sequence,
    NotATree as NotATree,
    to_nested_tuple as to_nested_tuple,
    to_prufer_sequence as to_prufer_sequence,
)
from .tree.decomposition import junction_tree as junction_tree
from .tree.mst import (
    minimum_spanning_edges as minimum_spanning_edges,
    maximum_spanning_edges as maximum_spanning_edges,
    minimum_spanning_tree as minimum_spanning_tree,
    maximum_spanning_tree as maximum_spanning_tree,
    random_spanning_tree as random_spanning_tree,
    partition_spanning_tree as partition_spanning_tree,
    EdgePartition as EdgePartition,
    SpanningTreeIterator as SpanningTreeIterator,
)
from .tree.operations import join as join
from .tree.recognition import (
    is_arborescence as is_arborescence,
    is_branching as is_branching,
    is_forest as is_forest,
    is_tree as is_tree,
)



from skimage._typing import Scalar
from numpy.typing import ArrayLike, NDArray
from typing import Literal
from collections.abc import Iterable

import numpy as np
from scipy import ndimage as ndi

from .._shared import utils
from .._shared.utils import _supported_float_type, convert_to_float, warn

@utils.deprecate_multichannel_kwarg(multichannel_position=5)
def gaussian(
    image: ArrayLike,
    sigma=1,
    output: ArrayLike | None = None,
    mode: Literal["reflect", "constant", "nearest", "mirror", "wrap"] = "nearest",
    cval: Scalar = 0,
    multichannel=None,
    preserve_range: bool = False,
    truncate: float = 4.0,
    *,
    channel_axis: int | None = None
) -> NDArray: ...



import numpy as np
from typing import Literal, overload
from matplotlib._typing import *
from .trifinder import TriFinder
from .triangulation import Triangulation

__all__ = ("TriInterpolator", "LinearTriInterpolator", "CubicTriInterpolator")

class TriInterpolator:
    def __init__(
        self, triangulation: Triangulation, z: ArrayLike, trifinder: TriFinder = ...
    ) -> None: ...

class LinearTriInterpolator(TriInterpolator):
    def __init__(
        self,
        triangulation: Triangulation,
        z: ArrayLike,
        trifinder: TriFinder = ...,
    ) -> None: ...
    def __call__(self, x: ArrayLike, y: ArrayLike) -> np.ndarray: ...
    def gradient(self, x: ArrayLike, y: ArrayLike) -> list: ...

class CubicTriInterpolator(TriInterpolator):
    def __init__(
        self,
        triangulation: Triangulation,
        z: ArrayLike,
        kind: Literal["min_E", "geom", "user"] = ...,
        trifinder: TriFinder = ...,
        dz: tuple = ...,
    ) -> None: ...
    def __call__(self, x: ArrayLike, y: ArrayLike) -> np.ndarray: ...
    def gradient(self, x: ArrayLike, y: ArrayLike)-> list: ...

class _ReducedHCT_Element:

    M: np.ndarray = ...
    M0: np.ndarray = ...
    M1: np.ndarray = ...
    M2: np.ndarray = ...
    rotate_dV: np.ndarray = ...
    rotate_d2V: np.ndarray = ...
    n_gauss = ...
    gauss_pts: np.ndarray = ...
    gauss_w: np.ndarray = ...
    E: np.ndarray = ...
    J0_to_J1: np.ndarray = ...
    J0_to_J2: np.ndarray = ...

    def get_function_values(
        self, alpha: np.ndarray, ecc: np.ndarray, dofs: np.ndarray
    ) -> np.ndarray: ...
    def get_function_derivatives(
        self, alpha: np.ndarray, J: np.ndarray, ecc: np.ndarray, dofs: np.ndarray
    ) -> np.ndarray: ...
    def get_function_hessians(
        self, alpha: np.ndarray, J: np.ndarray, ecc: np.ndarray, dofs: np.ndarray
    ) -> np.ndarray: ...
    def get_d2Sidksij2(self, alpha: np.ndarray, ecc: np.ndarray): ...
    def get_bending_matrices(self, J: np.ndarray, ecc: np.ndarray): ...
    @overload
    def get_Hrot_from_J(
        self, J: np.ndarray, return_area: Literal[True]
    ) -> tuple[np.ndarray, np.ndarray]: ...
    @overload
    def get_Hrot_from_J(self, J: np.ndarray, return_area=...) -> np.ndarray: ...
    def get_Kff_and_Ff(
        self, J: np.ndarray, ecc: np.ndarray, triangles: np.ndarray, Uc: np.ndarray
    ) -> tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray]: ...

class _DOF_estimator:
    def __init__(self, interpolator, **kwargs) -> None: ...
    def compute_dz(self, **kwargs): ...
    def compute_dof_from_df(self) -> np.ndarray: ...
    @staticmethod
    def get_dof_vec(
        tri_z: np.ndarray, tri_dz: np.ndarray, J: np.ndarray
    ) -> np.ndarray: ...

class _DOF_estimator_user(_DOF_estimator):
    def compute_dz(self, dz): ...

class _DOF_estimator_geom(_DOF_estimator):
    def compute_dz(self) -> np.ndarray: ...
    def compute_geom_weights(self) -> np.ndarray: ...
    def compute_geom_grads(self) -> np.ndarray: ...

class _DOF_estimator_min_E(_DOF_estimator_geom):
    def __init__(self, Interpolator) -> None: ...
    def compute_dz(self) -> np.ndarray: ...

class _Sparse_Matrix_coo:
    def __init__(
        self, vals: np.ndarray, rows: np.ndarray, cols: np.ndarray, shape: tuple
    ) -> None: ...
    def dot(self, V) -> np.ndarray: ...
    def compress_csc(self) -> None: ...
    def compress_csr(self) -> None: ...
    def to_dense(self) -> np.ndarray: ...
    def __str__(self) -> str: ...
    @property
    def diag(self) -> np.ndarray: ...



from typing import Mapping
from numpy.typing import ArrayLike
from ._typing import Scalar
import warnings
from collections.abc import Collection, Generator, Iterator

from .classes.graph import Graph

__all__ = [
    "to_networkx_graph",
    "from_dict_of_dicts",
    "to_dict_of_dicts",
    "from_dict_of_lists",
    "to_dict_of_lists",
    "from_edgelist",
    "to_edgelist",
]

def to_networkx_graph(data, create_using=None, multigraph_input=False): ...
def to_dict_of_lists(G: Graph, nodelist: ArrayLike | None = None): ...
def from_dict_of_lists(d, create_using=None): ...
def to_dict_of_dicts(
    G: Graph, nodelist: ArrayLike | None = None, edge_data: Scalar | None = None
) -> Mapping: ...
def from_dict_of_dicts(
    d: dict[dict, dict], create_using=None, multigraph_input=False
): ...
def to_edgelist(G: Graph, nodelist: ArrayLike | None = None): ...
def from_edgelist(edgelist, create_using=None): ...



"""
This type stub file was generated by pyright.
"""

import argparse
from IPython.utils.decorators import undoc

''' A decorator-based method of constructing IPython magics with `argparse`
option handling.

New magic functions can be defined like so::

    from IPython.core.magic_arguments import (argument, magic_arguments,
        parse_argstring)

    @magic_arguments()
    @argument('-o', '--option', help='An optional argument.')
    @argument('arg', type=int, help='An integer positional argument.')
    def magic_cool(self, arg):
        """ A really cool magic command.

    """
        args = parse_argstring(magic_cool, arg)
        ...

The `@magic_arguments` decorator marks the function as having argparse arguments.
The `@argument` decorator adds an argument using the same syntax as argparse's
`add_argument()` method. More sophisticated uses may also require the
`@argument_group` or `@kwds` decorator to customize the formatting and the
parsing.

Help text for the magic is automatically generated from the docstring and the
arguments::

    In[1]: %cool?
        %cool [-o OPTION] arg
        
        A really cool magic command.
        
        positional arguments:
          arg                   An integer positional argument.
        
        optional arguments:
          -o OPTION, --option OPTION
                                An optional argument.

Here is an elaborated example that uses default parameters in `argument` and calls the `args` in the cell magic::

    from IPython.core.magic import register_cell_magic
    from IPython.core.magic_arguments import (argument, magic_arguments,
                                            parse_argstring)


    @magic_arguments()
    @argument(
        "--option",
        "-o",
        help=("Add an option here"),
    )
    @argument(
        "--style",
        "-s",
        default="foo",
        help=("Add some style arguments"),
    )
    @register_cell_magic
    def my_cell_magic(line, cell):
        args = parse_argstring(my_cell_magic, line)
        print(f"{args.option=}")
        print(f"{args.style=}")
        print(f"{cell=}")

In a jupyter notebook, this cell magic can be executed like this::

    %%my_cell_magic -o Hello
    print("bar")
    i = 42

Inheritance diagram:

.. inheritance-diagram:: IPython.core.magic_arguments
   :parts: 3

'''
NAME_RE = ...
@undoc
class MagicHelpFormatter(argparse.RawDescriptionHelpFormatter):
    """A HelpFormatter with a couple of changes to meet our needs.
    """
    def add_usage(self, usage, actions, groups, prefix=...): # -> None:
        ...
    


class MagicArgumentParser(argparse.ArgumentParser):
    """ An ArgumentParser tweaked for use by IPython magics.
    """
    def __init__(self, prog=..., usage=..., description=..., epilog=..., parents=..., formatter_class=..., prefix_chars=..., argument_default=..., conflict_handler=..., add_help=...) -> None:
        ...
    
    def error(self, message):
        """ Raise a catchable error instead of exiting.
        """
        ...
    
    def parse_argstring(self, argstring):
        """ Split a string into an argument list and parse that argument list.
        """
        ...
    


def construct_parser(magic_func): # -> MagicArgumentParser:
    """ Construct an argument parser using the function decorations.
    """
    ...

def parse_argstring(magic_func, argstring):
    """ Parse the string of arguments for the given magic function.
    """
    ...

def real_name(magic_func):
    """ Find the real name of the magic.
    """
    ...

class ArgDecorator:
    """ Base class for decorators to add ArgumentParser information to a method.
    """
    def __call__(self, func):
        ...
    
    def add_to_parser(self, parser, group): # -> None:
        """ Add this object's information to the parser, if necessary.
        """
        ...
    


class magic_arguments(ArgDecorator):
    """ Mark the magic as having argparse arguments and possibly adjust the
    name.
    """
    def __init__(self, name=...) -> None:
        ...
    
    def __call__(self, func):
        ...
    


class ArgMethodWrapper(ArgDecorator):
    """
    Base class to define a wrapper for ArgumentParser method.

    Child class must define either `_method_name` or `add_to_parser`.

    """
    _method_name: str
    def __init__(self, *args, **kwds) -> None:
        ...
    
    def add_to_parser(self, parser, group): # -> None:
        """ Add this object's information to the parser.
        """
        ...
    


class argument(ArgMethodWrapper):
    """ Store arguments and keywords to pass to add_argument().

    Instances also serve to decorate command methods.
    """
    _method_name = ...


class defaults(ArgMethodWrapper):
    """ Store arguments and keywords to pass to set_defaults().

    Instances also serve to decorate command methods.
    """
    _method_name = ...


class argument_group(ArgMethodWrapper):
    """ Store arguments and keywords to pass to add_argument_group().

    Instances also serve to decorate command methods.
    """
    def add_to_parser(self, parser, group):
        """ Add this object's information to the parser.
        """
        ...
    


class kwds(ArgDecorator):
    """ Provide other keywords to the sub-parser constructor.
    """
    def __init__(self, **kwds) -> None:
        ...
    
    def __call__(self, func):
        ...
    


__all__ = ['magic_arguments', 'argument', 'argument_group', 'kwds', 'parse_argstring']



# COMPLETE

from typing import (BinaryIO, List, Literal, NamedTuple, Optional, Tuple,
                    Union, overload)


class CharMetrics(NamedTuple):
    width: float
    name: str
    bbox: List[int]

class CompositePart(NamedTuple):
    name: str
    dx: float
    dy: float

class AFM:
    def __init__(self, fh: BinaryIO) -> None: ...

    @property
    def postscript_name(self) -> str: ...
    @property
    def family_name(self) -> str: ...

    def get_angle(self) -> float: ...

    @overload
    def get_bbox_char(self, c: str, isord: Literal[False] = ...) -> List[int]: ...
    @overload
    def get_bbox_char(self, c: int, isord: Literal[True] = ...) -> List[int]: ...
    @overload
    def get_bbox_char(self, c: Union[str, int], isord: bool = ...) -> List[int]: ...

    def get_capheight(self) -> float: ...

    def get_familyname(self) -> str: ...
    def get_fontname(self) -> str: ...
    def get_fullname(self) -> str: ...
    
    @overload
    def get_height_char(self, c: str, isord: Literal[False] = ...) -> int: ...
    @overload
    def get_height_char(self, c: int, isord: Literal[True] = ...) -> int: ...
    @overload
    def get_height_char(self, c: Union[str, int], isord: bool = ...) -> int: ...

    def get_horizontal_stem_width(self) -> float: ...

    def get_kern_dist(self, c1: Union[str, int], c2: Union[str, int]) -> float: ...
    def get_kern_dist_from_name(self, name1: str, name2: str) -> float: ...

    @overload
    def get_name_char(self, c: str, isord: Literal[False] = ...) -> str: ...
    @overload
    def get_name_char(self, c: int, isord: Literal[True] = ...) -> str: ...
    @overload
    def get_name_char(self, c: Union[str, int], isord: bool = ...) -> str: ...
    
    def get_str_bbox(self, s: Union[str, bytes]) -> Tuple[float, float, float, float, float]: ...
    def get_str_bbox_and_descent(self, s: Union[str, bytes]) -> Tuple[float, float, float, float, float]: ...
    def get_underline_thickness(self) -> float: ...
    def get_vertical_stem_width(self) -> Optional[float]: ...
    def get_weight(self) -> str: ...

    @overload
    def get_width_char(self, c: str, isord: Literal[False] = ...) -> float: ...
    @overload
    def get_width_char(self, c: int, isord: Literal[True] = ...) -> float: ...
    @overload
    def get_width_char(self, c: Union[str, int], isord: bool = ...) -> float: ...

    def get_width_from_char_name(self, name: str) -> float: ...

    def get_xheight(self) -> float: ...

    def string_width_height(self, s: str) -> Tuple[float, float]: ...



from typing import Any

from django.db import models

class Redirect(models.Model):
    site: models.ForeignKey[Any] = ...
    old_path: models.CharField[Any] = ...
    new_path: models.CharField[Any] = ...



__all__ = [
    "generate_multiline_adjlist",
    "write_multiline_adjlist",
    "parse_multiline_adjlist",
    "read_multiline_adjlist",
]

from ..classes.graph import Graph
from ..utils import open_file

def generate_multiline_adjlist(G: Graph, delimiter: str = " ") -> str: ...
def write_multiline_adjlist(
    G: Graph, path, delimiter: str = " ", comments: str = "#", encoding: str = "utf-8"
): ...
def parse_multiline_adjlist(
    lines,
    comments: str = "#",
    delimiter: str | None = None,
    create_using=None,
    nodetype=None,
    edgetype=None,
): ...
def read_multiline_adjlist(
    path,
    comments: str = "#",
    delimiter: str | None = None,
    create_using=None,
    nodetype=None,
    edgetype=None,
    encoding="utf-8",
): ...



from typing import Mapping
from numpy.typing import ArrayLike

# See https://github.com/networkx/networkx/pull/1474
# Copyright 2011 Reya Group <http://www.reyagroup.com>
# Copyright 2011 Alex Levenson <alex@isnotinvain.com>
# Copyright 2011 Diederik van Liere <diederik.vanliere@rotman.utoronto.ca>

from collections import defaultdict
from itertools import combinations, permutations
from random import sample

from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = [
    "triadic_census",
    "is_triad",
    "all_triplets",
    "all_triads",
    "triads_by_type",
    "triad_type",
    "random_triad",
]

#: The integer codes representing each type of triad.
#:
#: Triads that are the same up to symmetry have the same code.
TRICODES = ...

#: The names of each type of triad. The order of the elements is
#: important: it corresponds to the tricodes given in :data:`TRICODES`.
TRIAD_NAMES = ...

#: A dictionary mapping triad code to triad name.
TRICODE_TO_NAME: dict = ...

def triadic_census(G: Graph, nodelist: ArrayLike | None = None) -> Mapping: ...
def is_triad(G: Graph) -> bool: ...
def all_triplets(G: Graph): ...
def all_triads(G: Graph): ...
def triads_by_type(G: Graph) -> Mapping: ...
def triad_type(G: Graph) -> str: ...
def random_triad(G: Graph): ...




class CyLossFunction:
    def cy_loss(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_gradient(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_grad_hess(self, y_true: float, raw_prediction: float)->tuple[float,float]: ...


class CyHalfSquaredError(CyLossFunction):
    def cy_loss(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_gradient(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_grad_hess(self, y_true: float, raw_prediction: float)->tuple[float,float]: ...


class CyAbsoluteError(CyLossFunction):
    def cy_loss(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_gradient(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_grad_hess(self, y_true: float, raw_prediction: float)->tuple[float,float]: ...


class CyPinballLoss(CyLossFunction):
    quantile: float
    def cy_loss(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_gradient(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_grad_hess(self, y_true: float, raw_prediction: float)->tuple[float,float]: ...


class CyHalfPoissonLoss(CyLossFunction):
    def cy_loss(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_gradient(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_grad_hess(self, y_true: float, raw_prediction: float)->tuple[float,float]: ...


class CyHalfGammaLoss(CyLossFunction):
    def cy_loss(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_gradient(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_grad_hess(self, y_true: float, raw_prediction: float)->tuple[float,float]: ...


class CyHalfTweedieLoss(CyLossFunction):
    power: float
    def cy_loss(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_gradient(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_grad_hess(self, y_true: float, raw_prediction: float)->tuple[float,float]: ...


class CyHalfTweedieLossIdentity(CyLossFunction):
    power: float
    def cy_loss(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_gradient(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_grad_hess(self, y_true: float, raw_prediction: float)->tuple[float,float]: ...

    
class CyHalfBinomialLoss(CyLossFunction):
    def cy_loss(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_gradient(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_grad_hess(self, y_true: float, raw_prediction: float)->tuple[float,float]: ...

    
class CyHalfMultinomialLoss(CyLossFunction):
    def cy_loss(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_gradient(self, y_true: float, raw_prediction: float) -> float: ...
    def cy_grad_hess(self, y_true: float, raw_prediction: float)->tuple[float,float]: ...





from ._random import sample_without_replacement
from . import check_random_state as check_random_state

# Author: Hamzeh Alsalhi <ha258@cornell.edu>
#
# License: BSD 3 clause
import numpy as np
import scipy.sparse as sp
import array

__all__ = ["sample_without_replacement"]



"""
This type stub file was generated by pyright.
"""

from prompt_toolkit.completion import Completer
from prompt_toolkit.lexers import Lexer

"""prompt-toolkit utilities

Everything in this module is a private API,
not to be used outside IPython.
"""
_completion_sentinel = ...
class IPythonPTCompleter(Completer):
    """Adaptor to provide IPython completions to prompt_toolkit"""
    def __init__(self, ipy_completer=..., shell=...) -> None:
        ...
    
    @property
    def ipy_completer(self):
        ...
    
    def get_completions(self, document, complete_event):
        ...
    


class IPythonPTLexer(Lexer):
    """
    Wrapper around PythonLexer and BashLexer.
    """
    def __init__(self) -> None:
        ...
    
    def lex_document(self, document):
        ...
    





from typing import ClassVar, TypeVar
from numpy.random import RandomState
from scipy.special import expit as expit
from numpy import ndarray
from ..utils.extmath import (
    safe_sparse_dot as safe_sparse_dot,
    log_logistic as log_logistic,
)
from ..utils._param_validation import Interval as Interval
from numbers import Integral as Integral, Real as Real
from ..base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from .._typing import Int, Float, MatrixLike, ArrayLike
from ..utils import (
    check_random_state as check_random_state,
    gen_even_slices as gen_even_slices,
)
from ..utils.validation import check_is_fitted as check_is_fitted

BernoulliRBM_Self = TypeVar("BernoulliRBM_Self", bound="BernoulliRBM")


# Authors: Yann N. Dauphin <dauphiya@iro.umontreal.ca>
#          Vlad Niculae
#          Gabriel Synnaeve
#          Lars Buitinck
# License: BSD 3 clause

import time

import numpy as np
import scipy.sparse as sp


class BernoulliRBM(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    h_samples_: ArrayLike = ...
    components_: ArrayLike = ...
    intercept_visible_: ArrayLike = ...
    intercept_hidden_: ArrayLike = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: Int = 256,
        *,
        learning_rate: Float = 0.1,
        batch_size: Int = 10,
        n_iter: Int = 10,
        verbose: Int = 0,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def gibbs(self, v: ArrayLike) -> ndarray:
        ...

    def partial_fit(
        self: BernoulliRBM_Self, X: ArrayLike, y: None | MatrixLike | ArrayLike = None
    ) -> BernoulliRBM_Self:
        ...

    def score_samples(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def fit(
        self: BernoulliRBM_Self,
        X: MatrixLike | ArrayLike,
        y: None | MatrixLike | ArrayLike = None,
    ) -> BernoulliRBM_Self:
        ...



from itertools import islice

from ...classes.graph import Graph

from ...utils import BinaryHeap, arbitrary_element, not_implemented_for

__all__ = ["stoer_wagner"]

def stoer_wagner(G: Graph, weight: str = "weight", heap=...): ...



from typing import Mapping
from numpy.typing import ArrayLike
from warnings import warn

from ...classes.graph import Graph

__all__ = ["pagerank", "pagerank_numpy", "pagerank_scipy", "google_matrix"]

def pagerank(
    G: Graph,
    alpha: float = 0.85,
    personalization: Mapping | None = None,
    max_iter=100,
    tol: float = 1.0e-6,
    nstart: Mapping | None = None,
    weight="weight",
    dangling: Mapping | None = None,
) -> Mapping: ...
def google_matrix(
    G: Graph,
    alpha: float = 0.85,
    personalization: Mapping | None = None,
    nodelist: ArrayLike | None = None,
    weight="weight",
    dangling: Mapping | None = None,
): ...
def pagerank_numpy(
    G: Graph,
    alpha: float = 0.85,
    personalization: Mapping | None = None,
    weight="weight",
    dangling: Mapping | None = None,
) -> Mapping: ...
def pagerank_scipy(
    G: Graph,
    alpha: float = 0.85,
    personalization: Mapping | None = None,
    max_iter=100,
    tol: float = 1.0e-6,
    nstart: Mapping | None = None,
    weight="weight",
    dangling: Mapping | None = None,
) -> Mapping: ...



from typing import ClassVar, Optional, Tuple, Union
from ._structures import (
    Infinity as Infinity,
    InfinityType as InfinityType,
    NegativeInfinity as NegativeInfinity,
    NegativeInfinityType as NegativeInfinityType,
)

# Copyright (c) Donald Stufft and individual contributors.
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

#     1. Redistributions of source code must retain the above copyright notice,
#        this list of conditions and the following disclaimer.

#     2. Redistributions in binary form must reproduce the above copyright
#        notice, this list of conditions and the following disclaimer in the
#        documentation and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import collections
import itertools
import re
import warnings

__all__ = ["parse", "Version", "LegacyVersion", "InvalidVersion", "VERSION_PATTERN"]

InfiniteTypes = ...
PrePostDevType = ...
SubLocalType = ...
LocalType = ...
CmpKey = ...
LegacyCmpKey = ...
VersionComparisonMethod = ...

_Version = ...


def parse(version: str) -> Union["LegacyVersion", "Version"]:
    ...


class InvalidVersion(ValueError):
    ...


class _BaseVersion:
    _key: ClassVar[Union[CmpKey, LegacyCmpKey]] = ...

    def __hash__(self) -> int:
        ...

    # Please keep the duplicated `isinstance` check
    # in the six comparisons hereunder
    # unless you find a way to avoid adding overhead function calls.
    def __lt__(self, other: "_BaseVersion") -> bool:
        ...

    def __le__(self, other: "_BaseVersion") -> bool:
        ...

    def __eq__(self, other: object) -> bool:
        ...

    def __ge__(self, other: "_BaseVersion") -> bool:
        ...

    def __gt__(self, other: "_BaseVersion") -> bool:
        ...

    def __ne__(self, other: object) -> bool:
        ...


class LegacyVersion(_BaseVersion):
    def __init__(self, version: str) -> None:
        ...

    def __str__(self) -> str:
        ...

    def __repr__(self) -> str:
        ...

    def public(self) -> str:
        ...

    def base_version(self) -> str:
        ...

    def epoch(self) -> int:
        ...

    def release(self) -> None:
        ...

    def pre(self) -> None:
        ...

    def post(self) -> None:
        ...

    def dev(self) -> None:
        ...

    def local(self) -> None:
        ...

    def is_prerelease(self) -> bool:
        ...

    def is_postrelease(self) -> bool:
        ...

    def is_devrelease(self) -> bool:
        ...


_legacy_version_component_re = ...

_legacy_version_replacement_map: dict = ...


# Deliberately not anchored to the start and end of the string, to make it
# easier for 3rd party code to reuse
VERSION_PATTERN: str = ...


class Version(_BaseVersion):

    _regex = ...

    def __init__(self, version: str) -> None:
        ...

    def __repr__(self) -> str:
        ...

    def __str__(self) -> str:
        ...

    def epoch(self) -> int:
        ...

    def release(self) -> Tuple[int, ...]:
        ...

    def pre(self) -> Optional[Tuple[str, int]]:
        ...

    def post(self) -> Optional[int]:
        ...

    def dev(self) -> Optional[int]:
        ...

    def local(self) -> Optional[str]:
        ...

    def public(self) -> str:
        ...

    def base_version(self) -> str:
        ...

    def is_prerelease(self) -> bool:
        ...

    def is_postrelease(self) -> bool:
        ...

    def is_devrelease(self) -> bool:
        ...

    def major(self) -> int:
        ...

    def minor(self) -> int:
        ...

    def micro(self) -> int:
        ...


_local_version_separators = ...
